<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');

if (!mxGetAdminPref("radminsuper")) {
    mxErrorScreen("Access Denied");
    die();
}


class textarea_admin {

    /**
     * 
     */
    function __construct($op)
    {
        load_class('Textarea', false);
        load_class('Filebrowse', false);

        mxGetLangfile(__DIR__);

        // Was ist zu tun...
        switch($op) {
            case PMX_MODULE . '/save':
                self::saveEditor();
                break;
            default:
                self::readEditor();
                break;
        }
    }

    /**
     * ReadEditor()
     * Einstellungen Formular
     *
     * @return
     */
    function readEditor()
    {
        $wyscnf = Textarea::get_config();

        // Die Liste der pragmaMx Usergruppen
        $wyscnf['pmx_groups_list'] = self::_mxeditor_get_usergroups_list();
        if (count($wyscnf['pmx_groups_list']) > 1) {
            $usertitle = _USERSGROUPS;
        } else {
            $usertitle = _USERSMEMBERS;
        }

        $wyscnf['groups']['admin'] = array('title' => _EDITOR_FOR . ' ' . _USERSADMINS);
        $wyscnf['groups']['user'] = array('title' => _EDITOR_FOR . ' ' . $usertitle);
        $wyscnf['groups']['other'] = array('title' => _EDITOR_FOR . ' ' . _USERSOTHERS);

        $langfiletypes = array(// Übersetzung der Dateitypen
            'any' => _FILES_ANY,
            'images' => _FILES_IMAGES,
            'flash' => _FILES_FLASH,
            'documents' => _FILES_DOCUMENT,
            'audio' => _FILES_AUDIO,
            'video' => _FILES_VIDEO,
            'archives' => _FILES_ARCHIVES,
            );

        $filetypegroups = Textarea::get_filetypegroups();
        $filetype_options = array();
        foreach ($filetypegroups as $key => $value) {
            if (isset($langfiletypes[$key])) {
                $label = $langfiletypes[$key];
            } else {
                $label = ucfirst($key);
            }
            if ($value) {
                $types = ' <small>(.' . implode(' .', array_keys($value)) . ')</small>';
            } else {
                $types = '';
            }

            $filetype_labels[$key] = htmlspecialchars($label) . $types;
            foreach($wyscnf['groups'] as $group => $tmp) {
                $checked = (empty($wyscnf[$group]['filetype'][$key])) ? '': ' checked="checked"';
                $filetype_options[$group][$key] = '
                <input type="hidden" name="filetype[' . $group . '][' . $key . ']" value="0">
                <input type="checkbox" name="filetype[' . $group . '][' . $key . ']" value="1"' . $checked . '>
                ';
            }
        }

        // all core modes
        $modes = Textarea::get_mode_levels();
        foreach ($modes as $key => $value) {
            $const = '_EDITOR_MODE_' . strtoupper($value);
            $modes[$key] = (defined($const)) ? constant($const) : ucfirst($value);
        }

        $tmp_editors = Textarea::get_available_editors();
        if (count($tmp_editors) === 2) {
            // nur der Spaw vorhanden
            foreach ($tmp_editors as $editor) {
                if (Textarea::fallback == $editor) {
                    $editors['0'] = _NO;
                } else {
                    $editors[$editor] = _YES;
                }
            }
        } else {
            // weitere Editoren vorhanden
            foreach ($tmp_editors as $editor) {
                if (Textarea::fallback == $editor) {
                    $editors['0'] = '- ' . _NO;
                } else {
                    $editors[$editor] = $editor;
                }
            }
        }

        $tmp_managers = pmxFilebrowse::get_available_managers();
        if (count($tmp_managers) === 1) {
            // nur der elfinder vorhanden
            $managers['0'] = _NO;
            $managers[$tmp_managers[0]] = _YES;
        } else {
            $managers['0'] = '- ' . _NO;
            // weitere Editoren vorhanden
            foreach ($tmp_managers as $manager) {
                $managers[$manager] = $manager;
            }
        }

        // die Standard-Chmods
        if (!isset($wyscnf['globals']['chmod_dir_to']) || $wyscnf['globals']['chmod_dir_to'] < 0) {
            $mode = self::_mxEditorGetChmods('chmod_dir_to');
            $wyscnf['globals']['chmod_dir_to'] = sprintf("%o", $mode);
        }
        $wyscnf['globals']['chmod_dir_to'] = (empty($wyscnf['globals']['chmod_dir_to'])) ? '' : sprintf('%04d', $wyscnf['globals']['chmod_dir_to']);

        if (!isset($wyscnf['globals']['chmod_to']) || $wyscnf['globals']['chmod_to'] < 0) {
            $mode = self::_mxEditorGetChmods('chmod_to');
            $wyscnf['globals']['chmod_to'] = sprintf("%o", $mode);
        }
        $wyscnf['globals']['chmod_to'] = (empty($wyscnf['globals']['chmod_to'])) ? '' : sprintf('%04d', $wyscnf['globals']['chmod_to']);

        pmxHeader::add_style_code('table.editortabs tr{vertical-align:top;}table.editortabs .group-hidecaption{display:none;}table.editortabs td.nowrap{white-space:nowrap;}');
        pmxHeader::add_script_code_docu(
            '$("#fhdshdstrh a.lineplus").click(function(){
                $("#fhdshdstrh tr:last").after("<tr>" + $("#fhdshdstrh tr:last").html() + "<\/tr>");
            });'
        );

        // START Ausgabe
        mxIncludeHeader();
        title(_EDITOR_SETTINGS);
        OpenTable();

        $_tabItems = array();
        echo '<form action="' . adminUrl(PMX_MODULE) . '" method="post">
        <input type="hidden" name="op" value="' . PMX_MODULE . '/save">';
        foreach ($wyscnf['groups'] as $group => $groupdata) {
            $_tabItems[] = array(
                // title
                $groupdata['title'],
                // content
                '<table class="form editortabs">
                <tr><td colspan="2"><h3 class="group-hidecaption">' . $groupdata['title'] . '</h3></td></tr>'
                . self::_getTableMultiUser($group, $wyscnf)
                . self::_getTableEditor($group, $editors, $wyscnf, $modes)
                . self::_getTableFileAndFolder($group, $managers, $wyscnf)
                . self::_getTableFileTypes($filetype_labels, $filetype_options, $group)
                . self::_getTableGroupSettings($wyscnf, $group)
                . self::_formButton()
            );
        }

        // push settingTab
        $_tabItems[] = array(
			// title
			_EDITOR_MISCOPT,
			// content
			'<table class="form editortabs">'
			. '<tr><td colspan="2"><h3 class="group-hidecaption">' . _EDITOR_MISCOPT . '</h3></td></tr>'
			. self::_getTabSettings($wyscnf)
			. self::_formButton()
        );

        echo pmxDom::addTabs(["items" => $_tabItems, "cookie" => "tab_editoradmin"]);
        echo '<form>';

        CloseTable();
        mxIncludeFooter();
    }

    /**
     * Get the tab of settings
     */
    private function _getTabSettings(array $wyscnf)
    {
      return '<tr>
          <td><label>' . _EDITOR_MAX_IMG_WIDTH . '</label></td>
          <td class="nowrap"><input type="text" size="6" maxlength="5" name="globals[max_img_width]" value="' . $wyscnf["globals"]["max_img_width"] . '" style="text-align: right">&nbsp;' . _EDITOR_INPIXEL . '</td>
        </tr>
        <tr>
          <td><label>' . _EDITOR_MAX_IMG_HEIGHT . '</label></td>
          <td class="nowrap"><input type="text" size="6" maxlength="5" name="globals[max_img_height]" value="' . $wyscnf["globals"]["max_img_height"] . '" style="text-align: right">&nbsp;' . _EDITOR_INPIXEL . '</td>
        </tr>
        <tr>
          <td><label>' . _EDITOR_CHMODFOLDER . '</label></td>
          <td><input type="text" size="5" maxlength="4" name="globals[chmod_dir_to]" value="' . $wyscnf["globals"]["chmod_dir_to"] . '" style="text-align: right"></td>
        </tr>
        <tr>
          <td><label>' . _EDITOR_CHMODFILES . '</label></td>
          <td><input type="text" size="5" maxlength="4" name="globals[chmod_to]" value="' . $wyscnf["globals"]["chmod_to"] . '" style="text-align: right"></td>
        </tr>
        <tr><td colspan="2"><hr></td></tr>
        <tr>
          <td><label>' . _EDITOR_AREAFORECOLOR . '</label></td>
          <td><input type="color" size="10" name="globals[area_foreground]" value="' . $wyscnf["globals"]["area_foreground"] . '"></td>
        </tr>
        <tr>
          <td><label>' . _EDITOR_AREABACKCOLOR . '</label></td>
          <td><input type="color" size="10" name="globals[area_background]" value="' . $wyscnf["globals"]["area_background"] . '"></td>
        </tr>';
    }

    /**
     * Check if exists more as one usergroups
     * 
     * @param string $group
     * @param array $wyscnf
     */
    private function _getTableMultiUser($group, $wyscnf)
    {
        $_out = array();
        if ($group == 'user' && count($wyscnf['pmx_groups_list']) > 1) {
            $_out[] = '<tr>
            <td><label>' . _EDITOR_WHICH_USERS . '</label></td>
            <td>
                <table class="blind" cellspacing="0" cellpadding="0">';
                foreach($wyscnf['pmx_groups_list'] as $key => $label) {
                $checked = ((empty($wyscnf['globals']['pmxgroups'])) || (isset($wyscnf['globals']['pmxgroups']) && !empty($wyscnf['globals']['pmxgroups'][$key]))) ? 'checked': '';
                $_out[] = '<tr>
                    <td><input type="hidden" name="pmxgroups[' . $key . ']" value="0"><input type="checkbox" name="pmxgroups[' . $key . ']" value="1" ' . $checked . '></td>
                    <td>&nbsp;' . $label . '</td>
                </tr>';
                }
                $_out[] = '</table>
            </td>
            </tr>
            <tr><td colspan="2"><hr></td></tr>';
        }
        return implode("", $_out);
    }

    /**
     * @param string $group
     * @param array $editors
     * @param array $wyscnf
     * @param array $modes
     */
    private function _getTableEditor($group, $editors, $wyscnf, $modes)
    {
        $_out = array();
        $_out[] = '<tr>
            <td width="40%"><label>' . _EDITOR_ACTIVE . '</label></td>
            <td><select name="editor[' . $group . ']">';
            foreach ($editors as $key => $value) {
                $selected = $wyscnf[$group]['editor'] === strval($key) ? 'class="current" selected': '';
                $_out[] = '<option value="' . $key . '" ' . $selected . '>' . $value . '</option>';
            }
            $_out[] = '</select></td>
        </tr>
        <tr>
            <td><label>' . _EDITOR_TOOLMODE . '</label></td>
            <td><select name="mode[' . $group . ']">';
            foreach ($modes as $key2 => $value2) {
                $selected2 = ($wyscnf[$group]['mode'] == $key2) ? 'class="current" selected': '';
                $_out[] = '<option value="' . $key2 . '" ' . $selected2 . '>' . $value2 . '</option>';
            }
            $_out[] = '</select></td>
        </tr>
        <tr><td colspan="2"><hr></td></tr>';
        return implode("", $_out);
    }

    /**
     * @param string $group
     * @param array $managers
     * @param array $wyscnf
     */
    private function _getTableFileAndFolder($group, $managers, $wyscnf)
    {
        $_out = array();
        // file
        $_out[] = '<tr>
            <td width="40%"><label>' . _FILEMAN_ACTIVE . '</label></td>
            <td><select name="manager[' . $group . ']">';
            foreach ($managers as $key => $value) {
            $selected = ($wyscnf[$group]['manager'] === $key) ? 'class="current" selected': '';
            $_out[] = '<option value="' . $key . '" ' . $selected . '>' . $value . '</option>';
            }
            $_out[] = '</select></td>
        </tr>';

        // folder
        $_out[] = '<tr>
            <td width="40%"><label>' . _FILEMAN_FOLDER . '</label></td>
            <td>';
            
            if ($group == 'admin') {
                $_out[] = '<table class="form border" id="fhdshdstrh">
                    <tr>
                    <th>' . _FILEMAN_FOLDER_PATH . '</th>
                    <th>' . _FILEMAN_FOLDER_ALIAS . '</th>
                    <th style="text-align:center"><a style="cursor:pointer;text-decoration:none" class="lineplus" title="' . _FILEMAN_FOLDER_ADDLINE . '">+</a></th>
                    </tr>';
                    foreach($wyscnf[$group]['roots'] as $alias => $path) {
                    if (!file_exists($path)) {
                        continue;
                    }
                    $_out[] = '<tr>
                        <td><input type="text" name="roots[' . $group . '][path][]" value="' . $path . '" size="30"></td>
                        <td colspan="2"><input type="text" name="roots[' . $group . '][alias][]" value="' . $alias . '" size="15"></td>
                    </tr>';
                    }
                    $_out[] = '<tr>
                    <td><input type="text" name="roots[' . $group . '][path][]" value="" size="30"></td>
                    <td colspan="2"><input type="text" name="roots[' . $group . '][alias][]" value="" size="15"></td>
                    </tr>
                </table>';
            }
            
            // all other (not admin)
            else {
                foreach($wyscnf[$group]['roots'] as $alias => $path) {
                    $_out[] = '<input type="text" name="roots[' . $group . ']" value="' . $path . '" size="40">';
                    break;
                } // nur den ersten Pfad verwenden
            } // endif
            $_out[] = '</td>
        </tr>';
        return implode("", $_out);
    }

    /**
     * @private
     * @param array $filetype_labels
     * @param array $filetype_options
     * @param string $group
     * @return string
     */
    private function _getTableFileTypes($filetype_labels, $filetype_options, $group)
    {
        $_out = array();
        $_out[] = '<tr>
            <td><label>' . _EDITOR_FILETYPES . '</label></td>
            <td>
            <table class="blind" cellspacing="0" cellpadding="0">';
            foreach($filetype_labels as $key => $label) {
                $_out[] = '<tr valign="top">
                <td>' . $filetype_options[$group][$key] . '</td>
                <td><label>' . $label . '</label></td>
                </tr>';
            }
            $_out[] = '</table>
            </td>
        </tr>';
        return implode("", $_out);
    }

    /**
     * @private
     * @param array $wyscnf
     * @param string $group
     * @return string
     */
    private function _getTableGroupSettings($wyscnf, $group)
    {
        $_out = array();
        $value = (empty($wyscnf[$group]['upload_max_size'])) ? '0': intval($wyscnf[$group]['upload_max_size']);
        $checked = (empty($wyscnf[$group]['allow_modify'])) ? '': 'checked';
        $_out[] = '<tr>
            <td><label>' . _EDITOR_MFS . '</label></td>
            <td class="nowrap"><input type="text" size="6" maxlength="5" name="upload_max_size[' . $group . ']" value="' . $value . '" style="text-align:right;">&nbsp;' . _EDITOR_INKB . '</td>
        </tr>
        <tr><td colspan="2"><span class="tiny">' . _EDITOR_MFS_NOTE . '</span></td></tr>
        <tr>
            <td><label>' . _EDITOR_ADMINFILES . '</label></td>
            <td>
            <input type="hidden" name="allow_modify[' . $group . ']" value="0">
            <input type="checkbox" name="allow_modify[' . $group . ']" value="1" ' . $checked . '>
            </td>
        </tr>';
        return implode("", $_out);
    }

    /**
     * @private
     * @return string
     */
    private function _formButton() {
        return '<tr><td colspan="2"><hr><input type="submit" value="' . _EDITOR_SAVE . '"></td></tr></table>';
    }

    /**
     * saveEditor()
     * Editorkonfiguration in Datei speichern
     *
     * @return void
     */
    private function saveEditor()
    {
        $wyscnf = Textarea::get_config();//"wysiwyg");
		$write = array();
        // $conf_file = $wyscnf['file'];

        $userconfig = load_class('Userconfig');

        // spezielle Benutzergruppen, falls welche vorhanden
        if (!empty($_POST['pmxgroups']) && is_array($_POST['pmxgroups'])) {
            $newgroups = $_POST['pmxgroups'];
        } else {
            // wenn nicht, gilt das alles nur fuer die Standardgruppe
            $newgroups = array($userconfig->default_group => 1);
        }

        // die Standard-Chmods
        if (!empty($_POST['globals']) && $_POST['globals']['chmod_dir_to']) {
            if ($_POST['globals']['chmod_dir_to'] < 0) {
                $mode = self::_mxEditorGetChmods('chmod_dir_to');
                $_POST['globals']['chmod_dir_to'] = sprintf("%o", $mode);
            }
        }

        if (!empty($_POST['globals']) && $_POST['globals']['chmod_to']) {
            if ($_POST['globals']['chmod_to'] < 0) {
                $mode = self::_mxEditorGetChmods('chmod_to');
                $_POST['globals']['chmod_to'] = sprintf("%o", $mode);
            }
        }

        $write['globals'] = array(// globals-array erstellen
            'pmxgroups' => $newgroups,
            'chmod_dir_to' => empty($_POST['globals']['chmod_dir_to']) ? '' : sprintf('%04d', intval($_POST['globals']['chmod_dir_to'])),
            'chmod_to' => empty($_POST['globals']['chmod_to']) ? '' : sprintf('%04d', intval($_POST['globals']['chmod_to'])),
            // die groessen
            'max_img_width' => empty($_POST['globals']['max_img_width']) ? '0' : $_POST['globals']['max_img_width'],
            'max_img_height' => empty($_POST['globals']['max_img_height']) ? '0' : $_POST['globals']['max_img_height'],
            // die Farben der Textarea
            'area_foreground' => empty($_POST['globals']['area_foreground']) ? '#000000' : $_POST['globals']['area_foreground'],
            'area_background' => empty($_POST['globals']['area_background']) ? '#ffffff' : $_POST['globals']['area_background'],
        );

        // Versuchen die upload_max_filesize aus der php.ini auszuwerten
        $check = Textarea::get_max_uploadsize();
        if ($check > 0) {
            $check = $check / 1024;
        }

        // Die Benutzer(gruppen) spezifischen Einstellungen
        if (!empty($_POST['editor'])) {
            foreach ($_POST['editor'] as $key => $value) {
                $group = array();
                $group['upload_max_size'] = intval($_POST['upload_max_size'][$key]);
                if ($check > 0 && ($group['upload_max_size'] === -1 || $group['upload_max_size'] > $check)) {
                    $group['upload_max_size'] = $check;
                }
                $group['editor'] = $value;
                $group['mode'] = $_POST['mode'][$key];
                $group['allow_modify'] = intval($_POST['allow_modify'][$key]);
                $group['filetype'] = $_POST['filetype'][$key];

                $group['manager'] = (is_numeric($_POST['manager'][$key])) ? false : $_POST['manager'][$key];

                
                if ($key == 'admin') {
                    foreach ($_POST['roots']['admin']['path'] as $ii => $path) {
                        if ($path && file_exists($path)) {
                            $alias = $_POST['roots']['admin']['alias'][$ii];
                            if (!$alias) {
                                $alias = basename($path);
                            }
                            $group['roots'][$alias] = $path;
                        }
                    }
                } else {
                
                    // alias = _FILES damit als Konstante verwendbar...
                    $group['roots']['_FILES'] = preg_match('/^[a-zA-Z0-9_]+$/i', $_POST['roots'][$key]) === 0 ? $key : $_POST['roots'][$key];
                }

                $write[$key] = $group; //serialize($group);
          	}
		}
		Textarea::set_config($write);
		
		mxRedirect(adminUrl(PMX_MODULE), _EDITOR_SETTINGSAVED, 1);
        return;
    }

    /**
     * _mxEditorGetChmods()
     * chmods automatisch ermitteln
     *
     * @param $mode
     * @return
     */
    private function _mxEditorGetChmods($mode)
    {
        // die Standard-Chmods
        if ($mode === 'chmod_dir_to') {
            if ($file = realpath(PMX_IMAGE_DIR . '/iupload')) {
                $mode = fileperms($file);
            } else if ($file = realpath(PMX_REAL_BASE_DIR . '/media/images')) {
                $mode = fileperms($file);
            } else {
                $mode = fileperms(__DIR__);
            }
        } else {
            $found = false;
            $endings = array('gif', 'png', 'jpg', 'jpeg');
            foreach ((array)glob(PMX_IMAGE_DIR . DS . 'admin' . DS . '*') as $image) {
                if ($image) {
                    $info = pathinfo($image);
                    if (isset($info['extension']) && in_array(strtolower($info['extension']), $endings)) {
                        $mode = fileperms($image);
                        $found = true;
                        break;
                    }
                }
            }
            if (!$found) {
                $mode = fileperms(__FILE__);
            }
        }
        $mode &= 0x1ff; # Remove the bits we don't need
        return $mode;
    }

    /**
     * _mxeditor_get_usergroups_list()
     * Die Liste der pragmaMx Usergruppen
     *
     * @return
     */
    private function _mxeditor_get_usergroups_list()
    {   
        global $prefix;
        $userconfig = load_class('Userconfig');
        $groups = array();

        $result = sql_system_query("SELECT access_id, access_title FROM " . $prefix . "_groups_access ORDER BY access_title");
        $groups[$userconfig->default_group] = MX_FIRSTGROUPNAME;
        while ($row = sql_fetch_assoc($result)) {
            $groups[$row['access_id']] = $row['access_title'];
        }
        return $groups;
    }

}
new textarea_admin($op);

?>