<?php
/**
 * This file is part of
 *
 * SiriusGallery
 *
 * for pragmamx (www.pragmamx.org)
 *
 * $Revision: 638 $
 * $Author: PragmaMx $
 * $Date: 2022-11-13 17:58:50 +0100 (So, 13. Nov 2022) $
 *
 */
 
defined('mxMainFileLoaded') or die('access denied');
   

	/* Ordner mit Backup Inhalten  */
 define('PMX_BACKUP_PATH', PMX_DYNADATA_DIR . '/backup/');

/**
 * 
 * DB BACKUP class
 * 
 * based on  
 * @author djunehor
 * @copyright  Copyright (c) 2019 djunehor.com
 * @link  http://github.com/db-backup-restore
 * @version 1.0
 * 
 * adapted and extended by pragmaMx Team 2022
 *
 *----------------------------------------------------------------------
 */
class pmxBackup {
    var $db; // 
    var $database; // 
    var $sqldir; //
	var $lang = [];
    
    private $ds = "\n";
    
    public $sqlContent = "";
    public $prefix;
    public $sqlEnd = ";";
    public $savefile = "";
    public $messages = array();
	private $_maxSize = 2048;

	/**
	 *
	 *
	 * @param string $host
	 * @param string $username
	 * @param string $password
	 * @param string $database
	 * @param string $charset
	 * @param string $lang
	 */
    function __construct( $lang = 'en') {
	    // $this->setLang($lang);
        $this->host = pmxBase::get('dbhost');
        $this->username = pmxBase::get('dbuname');
        $this->password = pmxBase::get('dbpass');
        $this->database = pmxBase::get('dbname');
		$this->prefix = pmxBase::get('prefix');
		$this->user_prefix = pmxBase::get('user_prefix');
        $this->charset = 'utf-8';
        //set_time_limit(0);
        //mxDebugFuncVars($this->prefix);


    }


    /**
     * It returns an array of all the tables in the database
     * 
     * @return An array of all the tables in the database.
     */
    function getTables() {
        $res = sql_system_query ( "SHOW TABLES LIKE '".$this->prefix."%'" );
        $tables = array ();
        while ( $row = sql_fetch_array ( $res ) ) {
            $tables [] = $row [0];
        }
		
		/* wenn Usertabellen unterschiedlichen Prefix */
		if ($this->user_prefix != $this->prefix) {
			$res = sql_system_query ( "SHOW TABLES LIKE '".$this->user_prefix."%'" );
			$tables = array ();
			while ( $row = sql_fetch_array ( $res ) ) {
				$tables [] = $row [0];
			}        
		}
		return $tables;
    }

  
    /**
     * It returns an array of all the tables in the database, with the table name as the key and the
     * table status as the value
     * 
     * @return An array of table names and their status.
     */
    function getTablesStatus() {
        $res = sql_system_query ( "SHOW TABLE STATUS LIKE '".$this->prefix."%'" );
        $tables = array ();
        while ( $row = sql_fetch_array ( $res ) ) {
            $tables [$row[0]] = $row ;
        }
		
		/* wenn Usertabellen unterschiedlichen Prefix */
		if ($this->user_prefix != $this->prefix) {
			$res = sql_system_query ( "SHOW TABLE STATUS LIKE '".$this->user_prefix."%'" );
			$tables = array ();
			while ( $row = sql_fetch_array ( $res ) ) {
				$tables [$row[0]] = $row ;
			}        
		}
		return $tables;
    }


	/**
	 *
	 *
	 * @param string $tablename
	 * @param $dir
	 * @param int $size
	 *
	 * @return bool
	 */
    function backup($filename, $dir = NULL,$tablename = NULL, $size = NULL) {
        
        
        if (file_exists('dynadata/backup') || file_exists('dynadata')) {
            $savepath = 'dynadata/backup';
        } else {
            $savepath =  __DIR__.'/../backup/';
        }        
        $dir = $dir ? normalizePath($dir) : $savepath;
        // 
        //mxDebugFuncvars($filename,$dir,$tablename,$size);
        if (!is_dir ( $dir )) {
            if (!mkdir ( $dir, 0777, true )) {
                $this->_showMsg( mxText('_CREATE_DIRECTORY_FAILED') );
            }
        }
        $size = $size ? $size : $this->_maxSize*1024;
        $sql = '';
        // 
        $fileoption="";
        $tables=array();
        switch (true) {
            case (is_array($tablename)):
                // direkt übergebene Tabellennamen inkl. Prefix
                $tables=$tablename;
                $fileoption="_selected";
                break;
            case ($tablename=="all"):
            case ($tablename==""):
            case ($tablename==NULL):
                $tables=$this->getTables();
                $fileoption="_all";
                break;
            case ($tablename=="sysonly"):
                $fileoption="_system";
                $tables=$this->_getSysTables();
                break;
            case ($tablename=="modonly"):
                $fileoption="_modules";
                $tables=$this->_getModuleTables();
                break;        
            case ($tablename=="useronly"):
                $fileoption="_users";
                $tables=$this->_getUserTables();
                break;        
            case ($tablename=="important"):
                $fileoption="_important";
                $tables=$this->_getImportantTables();
                break;     
                   
        }
        

        if ( !is_array($tables) ) {
           // mxDebugFuncVars($tables,$tablename,$fileoption);
            if(sql_num_rows(sql_system_query( "SHOW TABLES LIKE '".$tablename."'")) == 1) {
             } else {
                $this->_showMsg("$tablename => ".mxText('_TABLE_NOT_FOUND'),true);
               // return; //die(_TABLE_NOT_FOUND);
            }
            $this->_showMsg("$tablename => ".mxText('_BACKING_UP_TABLE'));
            $sql = $this->_retrieve ();

            $sql .= $this->_insert_table_structure ( $tablename );

            $data = sql_system_query ( "select * from " . $tablename );

            $filename .= "_" . $tablename;

            $num_fields = sql_num_fields ( $data );
            // 
            $p = 1;
            // 
            while ( $record = sql_fetch_array ( $data ) ) {
                // 
                $sql .= $this->_insert_record ( $tablename, $num_fields, $record );
                // 
                if (strlen ( $sql ) >= $size * 1024) {
                    $file = $filename . "_v" . $p . ".sql";
                    if ($this->_write_file ( $sql, $file, $dir )) {
                        $this->_showMsg($tablename . " - " . $p . " - " .$dir . $filename ."");
                    } else {
                        $this->_showMsg(" - " . $tablename . " - ",true);
                        return false;
                    }
                    // 
                    $p ++;
                    // 
                    $sql = "";
                }
            }
            // 
            unset($data, $record);
            // 
            if ($sql != "") {
                $file = $filename . "_v" . $p . ".sql";
                if ($this->_write_file ( $sql, $file, $dir )) {
                    $this->_showMsg( "- " . $tablename . " - " . $p . " - " .$dir . $file ."");
                } else {
                    $this->_showMsg(" - " . $p . " - ");
                    return false;
                }
				unset ($sql);
            }

        } else {
            //mxDebugFuncVars($tables,$fileoption);
            if ($tables1 = sql_system_query (  "show table status from `" . $this->database ."`")) {
                $this->_showMsg(mxText('_RETRIEVE_TABLES'));
            } else {
                $this->_showMsg(mxText('_RETRIEVE_TABLES_FAIL'));
                return;
            }
            
            $sql .= $this->_retrieve ();
            // 
            $filename .= "";
            $p = 1;
            
			foreach ($tables as $table) {
				set_time_limit(99);
				
				$tablename=$table;
                //mxDebugFuncvars($table);
                $sql .= $this->_insert_table_structure ( $tablename );
                $data = sql_system_query(  "SELECT * FROM " . $tablename );
                $num_fields = sql_num_fields ( $data );

                //
                while ( $record = sql_fetch_array ( $data ) ) {
                    
                    $sql .= $this->_insert_record ( $tablename, $num_fields, $record );
                    
                    if (strlen ( $sql ) >= $size * 1024) {

                        $file = $filename . "_v".$p . ".sql";
                        
                        if ($this->_write_file ( $sql, $file, $dir )) {
                            $this->_showMsg("-".mxText('_VOLUME')."-" . $p . "-".mxText('_BACKUP_COMPLETE')." [ ".$dir.$filename." ]");
                        } else {
                            $this->_showMsg(mxText('_VOLUME') . $p . " - ".mxText('_BACKUP_FAILED')."!",true);
                            return false;
                        }
                        
                        $p ++;
                        
                        $sql = "";
                    }
                }
            }
            // sql
            if ($sql != "") {
                $file = $filename . "_v".$p . ".sql";
                if ($this->_write_file ( $sql, $file, $dir )) {
                    $this->_showMsg("-".mxText('_VOLUME')."-" . $p . "-".mxText('_BACKUP_COMPLETE')." [ ".$dir.$filename." ]");
                } else {
                    $this->_showMsg(mxText('_VOLUME')."-" . $p . "-".mxText('_BACKUP_FAILED'),true);
                    return false;
                }
            }
        }
        return $file;
    }

    //
    /* A private function that is used to show messages. */
    private function _showMsg($msg, $err=false){
        $err = $err ? 'ERROR'.": " : '' ;
        $this->messages[]= $err . $msg;
        //flush();
        return;
    }

    /* A function that takes two parameters,  and . */
    function writeMsg($msg, $err=false){
        $this->_showMsg($msg, $err);
        return;
    }


    
    public function get_messages() {
        return $this->messages;
    }
    

    
    private function _retrieve() {
        
        
        $value = '';
        $value .= '--' . $this->ds;
        $value .= '-- PragmaMx Backup-System '. $this->ds;
        $value .= '-- Version : ' . PMX_VERSION_NUM. $this->ds;
        $value .= '-- MySQL database dump' . $this->ds;
        $value .= '-- based on BackUp class, By Djunehor. ' . $this->ds;
        $value .= '-- http://github.com/djunehor ' . $this->ds;
        $value .= '--' . $this->ds;
        $value .= '-- '.mxText('_HOST').': ' . $this->host . $this->ds;
        $value .= '-- '.mxText('_BUILD_DATE').': ' . date ( 'Y' ) . ' '.mxText('_YEAR').'  ' . date ( 'm' ) . ' '.mxText('_MONTH').' ' . date ( 'd' ) . ' '.mxText('_DAY').' ' . date ( 'H:i' ) . $this->ds;
        $value .= '-- MySQL: ' . sql_get_server_info () . $this->ds;
        $value .= '-- PHP: ' . phpversion () . $this->ds;
        $value .= $this->ds;
        $value .= '--' . $this->ds;
        $value .= '-- '.mxText('_DATABASE').': `' . $this->database . '`' . $this->ds;
        $value .= '-- Prefix : ' . $this->prefix . $this->ds;
        $value .= '--' . $this->ds . $this->ds;
        $value .= '-- -------------------------------------------------------';
        $value .= $this->ds . $this->ds;
        return $value;
    }

    /**
     *
     *
     * @param $table
     * @return string
     */
    private function _insert_table_structure($table) {
        $sql = '';
        $sql .= "--" . $this->ds;
        $sql .= "-- " . $table . $this->ds;
        $sql .= "--" . $this->ds . $this->ds;

        //
        $sql .= "DROP TABLE IF EXISTS `" . $table . '`' . $this->sqlEnd . $this->ds;
        //
        $res = sql_system_query(  'SHOW CREATE TABLE `' . $table . '`' );
        $row = sql_fetch_array ( $res );
        $sql .= $row [1];
        $sql .= $this->sqlEnd . $this->ds;
        //
        $sql .= $this->ds;
        $sql .= "--" . $this->ds;
        $sql .= "--  " . $table . $this->ds;
        $sql .= "--" . $this->ds;
        $sql .= $this->ds;
        return $sql;
    }
    

    /**
     *
     *
     * @param string $table
     * @param int $num_fields
     * @param array $record
     * @return string
     */
    private function _insert_record($table, $num_fields, $record) {
        //
        $insert = '';
        $comma = "";
        $insert .= "INSERT IGNORE INTO `" . $table . "` VALUES(";
        //
        for($i = 0; $i < $num_fields; $i ++) {
            $insert .= ($comma . "'" . sql_real_escape_string (  $record [$i] ) . "'");
            $comma = ",";
        }
        $insert .= ");" . $this->ds;
        return $insert;
    }

    /**
     *
     *
     * @param string $sql
     * @param string $filename
     * @param string $dir
     * @return boolean
     */
    private function _write_file($sql, $filename, $dir,$replace_prefix=false) {
        
        $dir = $dir ? $dir : './backup/';
        //
        $dir= PMX_DYNADATA_PATH . DS . "backup".DS;
        if (!is_dir ( $dir )) {
            mkdir ( $dir, 0777, true );
        }
        $sql=($replace_prefix)?str_replace(pmxBase::prefix(),"{prefix}",$sql):$sql;

        $re = true;
        if (!@$fp = fopen ( $dir . $filename, "w+" )) {
            $re = false;
            $this->_showMsg(mxText('_OPEN_SQL_FAIL'),true);
        }
        if (!@fwrite ( $fp, $sql )) {
            $re = false;
            $this->_showMsg(mxText('_WRITE_SQL_FAIL'),true);
        }
        if (!@fclose ( $fp )) {
            $re = false;
            $this->_showMsg(mxText('_CLOSE_SQL_FAIL'),true);
        }
        return $re;
    }

    /**
     *
     *
     * @param string $sqlfile
     */
    function restore($sqlfile) {
        //
        if (!file_exists ( $sqlfile )) {
            $this->_showMsg(mxText('_SQL_FILE_NOT_EXIST'),true);
            return false;
        }
        //$this->lock ( $this->database );
        //
        $sqlpath = pathinfo ( $sqlfile );
        $this->sqldir = $sqlpath ['dirname'];
        //
        $volume = explode ( "_v", $sqlfile );
        $volume_path = $volume [0];
		
        $this->_showMsg(mxText('_DO_NOT_REFRESH'));
        $this->_showMsg(mxText('_PLEASE_WAIT'));
        if (empty ( $volume [1] )) {
            $this->_showMsg ( mxText('_IMPORTING_SQL')."：" . $sqlfile);
            //
            if ($this->_import ( $sqlfile )) {
                $this->_showMsg( mxText('_DB_IMPORT_SUCCEED'));
            } else {
                 $this->_showMsg(mxText('_DB_IMPORT_FAILED'),true);
                return;
            }
        } else {
            //
            $volume_id = explode ( ".sq", $volume [1] );
            // $volume_id
            $volume_id = intval ( $volume_id [0] );
            while ( $volume_id ) {
                $tmpfile = $volume_path . "_v" . $volume_id . ".sql";
                //
                if (file_exists ( $tmpfile )) {
                    //
                    $this->_showMsg(mxText('_IMPORTING_VOLUME')." $volume_id ：" . $tmpfile);
                    if ($this->_import ( $tmpfile )) {

                    } else {
                        $volume_id = $volume_id ? $volume_id :1;
                        exit ( mxText('_IMPORTING_VOLUME')."：" . $tmpfile . ' '.mxText('_IMPORTING_VOLUME_FAILED') );
                    }
                } else {
                    $this->_showMsg(mxText('_IMPORTING_VOLUME_SUCCEED'));
                    return;
                }
                $volume_id ++;
            }
        }
        return $this->messages;
    }

    /**
     *
     *
     * @param string $sqlfile
     * @return boolean
     */
    private function _import($sqlfile) {
        //
        global $prefix;
        $sqls = array ();
        $f = fopen ( $sqlfile, "rb" );
        //
        $create_table = '';
        while ( !feof ( $f ) ) {
            //
            $line = fgets ( $f );
            // eventuell Prefix ersetzen mit aktuellem prefix
            $line = str_replace("{prefix}",pmxBase::prefix(),$line);

            if (substr($line,0,2)=="--") continue;
            //
            if (!preg_match ( '/;/', $line ) || preg_match ( '/ENGINE=/', $line )) {
                //
                $create_table .= $line;
                //
                
                if (preg_match ( '/ENGINE=/', $create_table)) {
                    //mxDebugFuncVars($create_table);
                    $this->_insert_into($create_table);
                    //
                    $create_table = '';
                }
                //
                continue;
            }
            //
            $this->_insert_into($line);
        }
        fclose ( $f );
        return true;
    }

    //
    private function _insert_into($sql){
        //
        if (!sql_system_query(  trim ( $sql ) )) {
            $this->_showMsg(sql_error (), true);
            return false;
        }
        return true;
    }

    public function optimize_table ($tablename)
    {
        $total_gain=0;
        $table=false;
        if ( $tablename ) {
            $result=sql_system_query( "SHOW TABLES LIKE '".$tablename."'");
            if(sql_num_rows($result) == 1) {
                $table = sql_fetch_assoc($result);
             } else {
                $this->_showMsg("$tablename => ".mxText('_TABLE_NOT_FOUND'),true);
                die(_TABLE_NOT_FOUND);
            }
            // ok Tabelle gefunden
            $total = ($table['Data_length'] + $table['Index_length']) / 1024 ;
            $table['size'] = round ($total, 3);
            if ($table['Data_free']) {
                sql_query("OPTIMIZE TABLE " . $table['Name']);
                $gain = $table['Data_free'] / 1024 ;
                $total_gain += $gain;
                $table['gain'] = round($gain, 3);
                $table['status'] = '<strong>' . _OPTIMIZED . '</strong>';
            } else {
                $table['gain'] = '--';
                $table['status'] = _ALREADYOPTIMIZED;
            }

        }
        return $table;
    }

    public function optimize_all ()
    {
        $total_gain = 0;
        $tablelist = $this->getTablesStatus();
        $tableNames = array();
        $tableslist = array();

        foreach ($tablelist as $table) {
            $total = ($table['Data_length'] + $table['Index_length']) / 1024 ;
            $table['size'] = round ($total, 3);
            if ($table['Data_free']) {
                sql_query("OPTIMIZE TABLE " . $table['Name']);
                $gain = $table['Data_free'] / 1024 ;
                $total_gain += $gain;
                $table['gain'] = round($gain, 3);
                $table['status'] = 1; // optimiert
            } else {
                $table['gain'] = '--';
                $table['status'] = 0;  // nicht optimiert
            }
            $tableslist[] = $table;
            $tableNames[] = $table['Name'];
        }        

        $total_gain = round ($total_gain, 3);
        return array('total_gain'=>$total_gain,
         'tablelist'=>$tableslist,
         'tableNames'=>$tableNames 
            );

    }


    private function _getSysTables()
    {
        // ** Systemtabellen

        $tables = array(
            'authors',
            'blocks',
            'captcher',
            'comments',
            'content',
            'content_log',
            'counter',
            'main',
            'menu',
            'message',
            'modules',
            'sys_config',
            'plugins',
        );

        return $this->_addPrefix($tables);
    }

    private function _getTempTables()
    {
        $tables = array(
            'sys_session',
            'content_log',
            'user_temptable',
            'visitors',
            'securelog',
        );
        return $this->_addPrefix($tables);
    }

    private function _getModuleTables()
    {
        $modtables=array();
        $tables = $this->getTables();
        $systables=$this->_getSysTables();
        $usertables=$this->_getUserTables();
        $temptables=$this->_getTempTables();
        $nottable=array_flip(array_merge($systables,$usertables,$temptables));
        //mxDebugFuncvars($nottable);
        foreach($tables as $table){
            if (!array_key_exists($table,$nottable)) $modtables[]=$table;
        }
        return $modtables;      
    }

    private function _getUserTables()
    {
        $tables = array(
            'users',
            'userpoints',
            'user_ban',
            'visitors',
            'users_temptable',
            'userguest',
            'groups_access',
            'groups_blocks',
            'groups_modules',
        );
        return $this->_addPrefix($tables);
    }

    private function _getImportantTables()
    {
        $modtables=array();
        $tables = $this->getTables();
        $temptables=$this->_getTempTables();
        $nottable=array_flip($temptables);
        //mxDebugFuncvars($nottable);
        foreach($tables as $table){
            if (!array_key_exists($table,$nottable)) $modtables[]=$table;
        }

        return $modtables;      
    }    

    private function _addPrefix($tables)
    {
        foreach ($tables as $table)
        {
            $tables2[]=$this->prefix."_".$table;
        }
        unset($tables);
        return $tables2;        
    }
    /*
     * -------------------------------end---------------------------------
     */

    //
    private function close() {
        
    }

    //
    private function lock($tablename, $op = "WRITE") {
        if (sql_system_query(  "lock tables " . $tablename . " " . $op ))
            return true;
        else
            return false;
    }

    //
    private function unlock() {
        if (sql_system_query(  "unlock tables" ))
            return true;
        else
            return false;
    }

    //
    function __destruct() {
        if($this->db){
            sql_system_query(  "unlock tables" );
            
        }
    }

}
