<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 */

/**
 * pmxZip
 *
 * @package pragmaMx
 * @author terraproject
 * @copyright Copyright (c) 2020
 * @access public
 */


/*
*/
if (!defined("mxMainFileLoaded")) die ("You can't access this file directly...");

class pmxZip extends ZipArchive
{

    private $config = array();
	private $zipfiles = array();
	private $zip=FALSE;
	
	
    /**
     *  __construct 
     *  
     *  @param [in] $parameter 
     *  @return 
     *  
     *  Date: 2020-03-01T21:41:27 
     *  created by : Olaf Herfurth 
     */
    public function __construct($parameter = null)
    {
			$this->_setDefault();
			
			$ret = false;
			if (file_exists($parameter)) {
				$ret=$this->open($parameter,ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE);
			}

		return $ret;
	}

	/**
     * Configuration
     *
     * @param mixed $name
     * @return boolean true = ok
     * @return boolean false=Error
     */
    public function _setDefault()
    {
        global $prefix;

		$this->__set('zipname', "");
		$this->__set('temppath', PMX_DYNADATA_PATH . "tmp/");
		$this->__set('indexfile', FALSE);
		
        return;
    }
	
   	/**
    *  __set 
    *  
    *  @param [in] $name  
    *  @param [in] $value 
    *  @return 
    *  
    *  Date: 2020-03-01T19:17:55 
    *  created by : Olaf Herfurth 
    */
    public function __set($name, $value = null)
    {
        /* in Array eintragen */
        if (is_array($name) && $value === null) {
            foreach ($name as $key => $value) {
                $this->config[$key] = $value;
            }
        } else {
            $this->config[$name] = $value;
        }
        return true;
    }
    /**
     *  __get 
     *  
     *  @param [in] $name 
     *  @return 
     *  
     *  Date: 2020-03-01T19:18:02 
     *  created by : Olaf Herfurth 
     */
    public function __get($name)
    {
        if (array_key_exists($name, $this->config)) return $this->config[$name];
        return false;
    }

	/**
	 *  open 
	 *  
	 *  @param [in] $zipfile 
	 *  @return 
	 *  
	 *  Date: 2020-03-02T09:33:56 
	 *  created by : Olaf Herfurth 
	 */
	public function openZip ($zipfile = '',$options = FALSE)
	{
		if (!file_exists($zipfile)) {
			trigger_error ($zipfile." - "._MODULEFILENOTFOUND);
			return false;
		}
		$res = $this->open($zipfile);
		if ($res === TRUE) {
			// nachsehen, ob die indexfile da ist.
			$stat = $this->statName($this->config['indexfile']);
			/* wenn nicht, dann false und raus */
			if ($stat===FALSE) 
			{
				trigger_error("not a valid file for pragmaMx : ".$zipfile);
				return false;
			}
			return true;
		} 
		return false;
	}

	/**
	 *  addPath 
	 *  
	 *  importiert einen kompletten Pfad in ZIP-File inkl. Unterverzeichnissen
	 *  
	 *  @param [in] $path 
	 *  @return 
	 *  
	 *  Date: 2020-03-02T15:35:35 
	 *  created by : Olaf Herfurth 
	 */
	function addCompletePath ($source, $dest = "")
	{
		switch (TRUE)
		{
			case !$source:
			case !is_dir($source):
			case !is_readable($source):
				return false;
		}

		$source = str_replace('\\', '/', ($source)); //realpath
		$dest = str_replace('\\', '/', ($dest)); //realpath

		if (is_dir($source) === true)
		{
			$files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

			foreach ($files as $file)
			{
				$file = str_replace('\\', '/', $file);

				// Ignore "." and ".." folders
				if( in_array(substr($file, strrpos($file, '/')+1), array('.', '..')) )
					continue;

				//$file = realpath($file);
				//mxDebugFuncVars($file);
				if (is_dir($file) === true)
				{
					
					$this->addEmptyDir(str_replace(dirname($source) . '/', '', $file . '/'));
				}
				else if (is_file($file) === true)
				{
					$this->addFromString(str_replace(dirname($source) . '/', '', $file), file_get_contents($file));
				}
			}
		}
		else if (is_file($source) === true)
		{
			$this->addFromString(basename($source), file_get_contents($source));
		}


		return $this->numFiles;
	}
	
	
    /**
     *  extractSubdirTo 
     *  
     *  extrahiert ein Unterverzeichnis aus dem ZIP-Archiv in einen Zielordner inkl aller Unterverzeichnisse
     *  
     *  @param [in] $destination -> relativ zur Root
     *  @param [in] $subdir      
     *  @return 
     *  
     *  Date: 2020-03-02T17:43:03 
     *  created by : Olaf Herfurth 
     */
	public function extractSubdirTo($destination, $subdir)
    {
      $errors = array();

      // Prepare dirs
      if (substr($destination, mb_strlen(DS, "UTF-8") * -1) != DS)    $destination .= DS;

	  //$destination = dirname($destination);
      $destination = str_replace(array("/", "\\"), DS, $destination);
      $subdir = str_replace(array("/", "\\"), "/", $subdir);

	  if (substr($destination, mb_strlen(DS, "UTF-8") * -1) != DS)    $destination .= DS;

      if (substr($subdir, -1) != "/") $subdir .= "/";
	  
		//mxDebugFuncVars($this->numFiles,$destination,$subdir);
      // Extract files
      for ($i = 0; $i < $this->numFiles; $i++)
      {
        $filename = $this->getNameIndex($i);


        if (substr($filename, 0, mb_strlen($subdir, "UTF-8")) == $subdir)
        {
          $relativePath = substr($filename, mb_strlen($subdir, "UTF-8"));
          $relativePath = str_replace(array("/", "\\"), DS, $relativePath);

          if (mb_strlen($relativePath, "UTF-8") > 0)
          {
            if (substr($filename, -1) == "/")  // Directory
            {
              // New dir
              if (!is_dir($destination . $relativePath))
                if (!@mkdir($destination . $relativePath, 0755, true))
                  $errors[$i] = $filename;
            }
            else
            {
              if (dirname($relativePath) != ".")
              {
                if (!is_dir($destination . dirname($relativePath)))
                {
					// New dir (for file)
					if (is_writable($destination . dirname($relativePath)))
					{
					  @mkdir($destination . dirname($relativePath), 0755, true);
					}
                }
              }

              // New file
              if (@file_put_contents($destination . $relativePath, $this->getFromIndex($i)) === false)
                $errors[$i] = $filename;
            }
          }
        }
      }

      return $errors;
    }
	
	/**
	 *  ZipStatusString 
	 *  
	 *  @param [in] $status 
	 *  @return 
	 *  
	 *  Date: 2020-03-01T19:18:14 
	 *  created by : Olaf Herfurth 
	 */
	function ZipStatusString( $status )
{
		switch( (int) $status )
		{
			case ZipArchive::ER_OK           : return 'N No error';
			case ZipArchive::ER_MULTIDISK    : return 'N Multi-disk zip archives not supported';
			case ZipArchive::ER_RENAME       : return 'S Renaming temporary file failed';
			case ZipArchive::ER_CLOSE        : return 'S Closing zip archive failed';
			case ZipArchive::ER_SEEK         : return 'S Seek error';
			case ZipArchive::ER_READ         : return 'S Read error';
			case ZipArchive::ER_WRITE        : return 'S Write error';
			case ZipArchive::ER_CRC          : return 'N CRC error';
			case ZipArchive::ER_ZIPCLOSED    : return 'N Containing zip archive was closed';
			case ZipArchive::ER_NOENT        : return 'N No such file';
			case ZipArchive::ER_EXISTS       : return 'N File already exists';
			case ZipArchive::ER_OPEN         : return 'S Can\'t open file';
			case ZipArchive::ER_TMPOPEN      : return 'S Failure to create temporary file';
			case ZipArchive::ER_ZLIB         : return 'Z Zlib error';
			case ZipArchive::ER_MEMORY       : return 'N Malloc failure';
			case ZipArchive::ER_CHANGED      : return 'N Entry has been changed';
			case ZipArchive::ER_COMPNOTSUPP  : return 'N Compression method not supported';
			case ZipArchive::ER_EOF          : return 'N Premature EOF';
			case ZipArchive::ER_INVAL        : return 'N Invalid argument';
			case ZipArchive::ER_NOZIP        : return 'N Not a zip archive';
			case ZipArchive::ER_INTERNAL     : return 'N Internal error';
			case ZipArchive::ER_INCONS       : return 'N Zip archive inconsistent';
			case ZipArchive::ER_REMOVE       : return 'S Can\'t remove file';
			case ZipArchive::ER_DELETED      : return 'N Entry has been deleted';

			default: return sprintf('Unknown status %s', $status );
		}
	}
}
?>
