<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 604 $
 * $Author: PragmaMx $
 * $Date: 2022-09-09 08:23:30 +0200 (Fr, 09. Sep 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');

define("FTP_NATIVE",FALSE); //pmxBase::mxFTPon()//

/*load_class ("Ftp",false);*/


/**
 * Setzen von File-Permissions (chmods)
 * DEPRECATED!! use mx_chmod() instead.
 *
 * @param string $file Angabe des Files dessen Berechtigungen geändert werden sollen (kopmletter
 * Pfad)
 * @param string $mode Modus lock ergibt chmod 444, unllock ergibt chmod 777)
 * @return bool Liefert true wenn die Aktion erfolgreich war, ansonsten false
 */
/* function mxChangeFilePerms($file, $mode)
{
	trigger_error('Use of deprecated phpnuke-function ' . __FUNCTION__ . '()', E_USER_WARNING);
    if (!file_exists($file)) {
        return false;
    }
    $err = false;
    if ($mode == 'lock') {
        mx_chmod($file, PMX_CHMOD_LOCK);
        $err = (is_writable($file));
    } else if ($mode == 'unlock') {
        mx_chmod($file, PMX_CHMOD_FULLUNOCK);
        $err = (!is_writable($file));
    }
    return (isset($php_errormsg) || $err) ? false : true;
} */

/**
 * mx_chmod()
 *
 * @param mixed $path
 * @param mixed $mode
 * @return
 */
function mx_chmod($path, $mode, $umask = true)
{
    if (!function_exists('chmod')) {
        return false;
    }

    if ($umask && !function_exists('umask')) {
        $umask = false;
    }

    if ($umask) {
        $old = umask(0);
    }

    pmxDebug::pause();
    $result = chmod($path, $mode);
    pmxDebug::restore();

    if ($umask) {
        umask($old);
    }

    return $result;
}

/**
 * mx_write_file()
 * Schreibt eine Datei ohne Berücksichtigung der Schreibberechtigung.
 *
 * @param mixed $filename
 * @param mixed $content
 * @param mixed $lock
 * @return
 */
function mx_write_file($filename, $content, $lock = false)
{
    $folder = dirname($filename);

    switch (true) {
        /* wenn der Ordner nicht existiert kann auch nix geschrieben werden */
        case !file_exists($folder):
            return false;

            /* wenn ordner nicht beschreibbar ist */
        case !is_writable($folder):
            /* aktuellen chmod des Ordners zwischenspeichern */
            $dirmode = fileperms($folder);
            /* versuchen beschreibbar zu machen */
            mx_chmod($folder, PMX_CHMOD_FULLUNOCK);
            clearstatcache();
    }

    /* wenn Datei bereits existiert, aber schreibgeschützt ist */
    if (file_exists($filename) && !is_writable($filename)) {
        /* versuchen beschreibbar zu machen */
        mx_chmod($filename, PMX_CHMOD_UNLOCK);
        clearstatcache();
    }

    switch (true) {
        case !file_exists($filename): // Datei existiert nicht
        case is_writable($filename): // oder ist vorhanden und beschreibbar
            /* Inhalte schreiben */
            $result = file_put_contents($filename, $content);
            break;
        default: // Datei existiert ist aber nicht beschreibbar
            $result = false;
    }

    /* nach dem schreiben schreibschützen ? */
    if ($lock) {
        mx_chmod($filename, PMX_CHMOD_LOCK);
    }

    /* Wenn chmod des Ordners geändert wurde */
    if (isset($dirmode)) {
        /* den alten chmod des Ordners wieder herstellen */
        mx_chmod($folder, $dirmode, false); // chmod ohne umask!!
    }

    return $result !== false;
}


/**
 *  bool mx_copy ( string $source , string $dest [, resource $context ] )
 *  
 *  @param [string] $source  Parameter_Description
 *  @param [string] $dest    Parameter_Description
 *  
 *  @return Return_Description
 *  
 *  @details Details
 */

function mx_copy($source, $dest, $context=NULL) 
{
	if (FTP_NATIVE) //FTP verwenden
	{
		return pmxFtp::fcopy($source,$dest);
		
	} else {
		return @copy($source,$dest);
	}
	
	
}

/**
 *  @brief Brief
 *  
 *  @param [st] $dir Parameter_Description
 *  @return Return_Description
 *  
 *  @details Details
 *  
 */
 
function mx_chdir($dir)
{
	return chdir($dir);
}

/**
 *  @brief Brief
 *  
 *  @param [st] $dir Parameter_Description
 *  @return Return_Description
 *  
 *  @details Details
 *  
 */
 
function mx_scandir($dir)
{
	return scandir($dir);
}	 
	
/**
	 *  @brief Brief
	 *  
	 *  @param [st] $filename Parameter_Description
	 *  @return Return_Description
	 *  
	 *  @details Details
 */
function mx_is_file( string $filename) 
{
	return is_file($filename);
}


function mx_file_exists(&$filename)
{
	//mxDebugFuncVars($filename);
	
	$fx = realpath($filename);
	if ($fx) return true;
	
	$filename= str_replace("/",DS,$filename);
	$fx = realpath($filename);
	if ($fx) return true;	
	
	$filename=$_SERVER['DOCUMENT_ROOT']."/".$filename;
	$fx=realpath($filename);
	//mxDebugFuncVars($fx,$filename);
	return realpath($fx);

	
}

/**
 *  mx_is_writable
 *  
 *  param $filename
 *  return 
 *  
 */
function mx_is_writable($filename)
{
	if (!file_exists($filename)) {
		return true;
	}

	$oldmode = false;
	$oldmode = fileperms($filename);
	if (FTP_NATIVE) {
			/* aktuellen chmod der Datei zwischenspeichern */
			$oldmode = fileperms($filename);
			/* versuchen beschreibbar zu machen */
			@mxFTP::chmod($filename, PMX_CHMOD_UNLOCK);
			//clearstatcache();		
	} else {
		if (!is_writable($filename)) {
			/* aktuellen chmod der Datei zwischenspeichern */
			$oldmode = fileperms($filename);
			/* versuchen beschreibbar zu machen */
			@chmod($filename, PMX_CHMOD_UNLOCK);
			//clearstatcache();
		}
	}
		$result = is_writable($filename);

		if ($oldmode !== false) {
			@chmod($filename, $oldmode);
		}
	
	return $result;
}

function mx_stat($file) {

/* 	
clearstatcache();
	$ss=@stat($file);
	if(!$ss) return false; //Couldnt stat file

	$ts=array(
	  0140000=>'ssocket',
	  0120000=>'llink',
	  0100000=>'-file',
	  0060000=>'bblock',
	  0040000=>'ddir',
	  0020000=>'cchar',
	  0010000=>'pfifo'
	);

	$p=$ss['mode'];
	$t=decoct($ss['mode'] & 0170000); // File Encoding Bit
	$str =(array_key_exists(octdec($t),$ts))?$ts[octdec($t)]{0}:'u';
	$str.=(($p&0x0100)?'r':'-').(($p&0x0080)?'w':'-');
	$str.=(($p&0x0040)?(($p&0x0800)?'s':'x'):(($p&0x0800)?'S':'-'));
	$str.=(($p&0x0020)?'r':'-').(($p&0x0010)?'w':'-');
	$str.=(($p&0x0008)?(($p&0x0400)?'s':'x'):(($p&0x0400)?'S':'-'));
	$str.=(($p&0x0004)?'r':'-').(($p&0x0002)?'w':'-');
	$str.=(($p&0x0001)?(($p&0x0200)?'t':'x'):(($p&0x0200)?'T':'-'));

	$s=array(
	'perms'=>array(
	  'umask'=>sprintf("%04o",@umask()),
	  'human'=>$str,
	  'octal1'=>sprintf("%o", ($ss['mode'] & 000777)),
	  'octal2'=>sprintf("0%o", 0777 & $p),
	  'decimal'=>sprintf("%04o", $p),
	  'fileperms'=>@fileperms($file),
	  'mode1'=>$p,
	  'mode2'=>$ss['mode']),

	'owner'=>array(
	  'fileowner'=>$ss['uid'],
	  'filegroup'=>$ss['gid'],
	  'owner'=>
	  (function_exists('posix_getpwuid'))?
	  @posix_getpwuid($ss['uid']):'',
	  'group'=>
	  (function_exists('posix_getgrgid'))?
	  @posix_getgrgid($ss['gid']):''
	  ),

	'file'=>array(
	  'filename'=>$file,
	  'realpath'=>(@realpath($file) != $file) ? @realpath($file) : '',
	  'dirname'=>@dirname($file),
	  'basename'=>@basename($file)
	  ),

	'filetype'=>array(
	  'type'=>substr($ts[octdec($t)],1),
	  'type_octal'=>sprintf("%07o", octdec($t)),
	  'is_file'=>@is_file($file),
	  'is_dir'=>@is_dir($file),
	  'is_link'=>@is_link($file),
	  'is_readable'=> @is_readable($file),
	  'is_writable'=> @is_writable($file)
	  ),
	 
	'device'=>array(
	  'device'=>$ss['dev'], //Device
	  'device_number'=>$ss['rdev'], //Device number, if device.
	  'inode'=>$ss['ino'], //File serial number
	  'link_count'=>$ss['nlink'], //link count
	  'link_to'=>($ss['type']=='link') ? @readlink($file) : ''
	  ),

	'size'=>array(
	  'size'=>$ss['size'], //Size of file, in bytes.
	  'blocks'=>$ss['blocks'], //Number 512-byte blocks allocated
	  'block_size'=> $ss['blksize'] //Optimal block size for I/O.
	  ),

	'time'=>array(
	  'mtime'=>$ss['mtime'], //Time of last modification
	  'atime'=>$ss['atime'], //Time of last access.
	  'ctime'=>$ss['ctime'], //Time of last status change
	  'accessed'=>@date('Y M D H:i:s',$ss['atime']),
	  'modified'=>@date('Y M D H:i:s',$ss['mtime']),
	  'created'=>@date('Y M D H:i:s',$ss['ctime'])
	  ),
	);

	clearstatcache();
	return $s; 
	*/
}

/**
 *  mx_rmdir
 *  
 *  löscht angegebenen Pfad rekursiv inkl. der enthaltenen Dateien....
 *  
 *  @param $src 
 *  @return $src
 *  
 */
function mx_rmdir($src) {
	$dir = opendir($src);
	while(false !== ( $file = readdir($dir)) ) {
		if (( $file != '.' ) && ( $file != '..' )) {
			$full = $src . '/' . $file;
			if ( is_dir($full) ) {
				mx_rmdir($full);
			}
			else {
				unlink($full);
			}
		}
	}
	closedir($dir);
	rmdir($src);
}

/**
 *  mx_copyFolder
 *  
 *  Kopiert einen kompletten Ordner ink. der Dateien an einen anderen Ort
 *  
 *  @param $source 		Quell-Pfad
 *  @param $dest 		Ziel-Pfad
 *  @param $recursive 	boolean
 *  @return 
 *  
 */
 function mx_copyFolder($source, $dest, $recursive = false)
{
    if (!is_dir($dest)) 
    { 
        mkdir($dest); 
	} 
    $handle = @opendir($source);
    if(!$handle) return false;
	
	// No slash on the end, please...
	if ($source !== '/' && substr($source, -1) === '/')	$source = substr($source, 0, -1);
	if ($dest   !== '/' && substr($dest, -1)   === '/')	$dest   = substr($dest, 0, -1);
    
	$ret['files']=0;
	$ret['dirs']=0;
	
    while ($file = @readdir ($handle))
    {
        if (preg_match("#^\.{1,2}$#i",$file))
        {
            continue;
        }
        if(!$recursive && $source != $source.$file."/")
        {
            if(is_dir($source.$file))
                continue;
        }
        if(is_dir($source.$file))
        {
            mx_copyFolder($source.$file."/", $dest.$file."/", $recursive);
			$ret['dirs'] ++;
        }
        else
        {
            copy($source.$file, $dest.$file);
			$ret['files'] ++;
        }
    }
    @closedir($handle);
	return $ret;
}

/**
 *  mx_normalize_files_array
 *  
 *  The problem occurs when you have a form that uses both single file and HTML array feature. 
 *  The array isn't normalized and tends to make coding for it really sloppy. 
 *  I have included a nice method to normalize the $_FILES array.
 *  input array $files - $_FILES
 *  @return array 
 *  
 *  sample:
 *  
 *  $files= mx_normalize_files_array($_FILES);
 *  
 */
 function mx_normalize_files_array($files = []) {

	$normalized_array = [];

	foreach($files as $index => $file) {

		if (!is_array($file['name'])) {
			$normalized_array[$index][] = $file;
			continue;
		}

		foreach($file['name'] as $idx => $name) {
			$normalized_array[$index][$idx] = [
				'name' => $name,
				'type' => $file['type'][$idx],
				'tmp_name' => $file['tmp_name'][$idx],
				'error' => $file['error'][$idx],
				'size' => $file['size'][$idx]
			];
		}

	}

	return $normalized_array;

}	
/**
 *  mx_file_put_contents 
 *  
 *  @param [in] $filename 
 *  @param [in] $data     
 *  @param [in] $flags    
 *  @param [in] $context  
 *  @return 
 *  
 *  Date: 2020-02-22T17:09:57 
 *  created by : Olaf Herfurth 
 */
function mx_file_put_contents ( $filename ,  $data , $flags = 0 , $context = false ) {
	
	return  file_put_contents ( $filename ,  $data , $flags , $context  );
}


/**
 *  mx_file_get_contents 
 *  
 *  @param [in] $filename 
 *  @param [in] $data     
 *  @param [in] $flags    
 *  @param [in] $context  
 *  @return 
 *  
 *  Date: 2020-02-22T17:09:57 
 *  created by : Olaf Herfurth 
 */
function mx_file_get_contents ( $filename ,  $include_path = false ,  $context = false, $offset=0, $maxlen=0 ) {
	
	return  file_get_contents ( $filename ,  $include_path   );
}

/**
 *  mx_deleteFolder - löscht rekursiv ein Verzeichnis
 *  
 *  @param [in] $dir - Pfad, welcher gelöscht werden soll 
 *  @return 
 *  
 *  Date: 2020-02-22T17:09:25 
 *  created by : Olaf Herfurth 
 */
function mx_deleteFolder($dir) {
	
	 // Wenn der Input ein Ordner ist, dann Überprüfung des Inhaltes beginnen
	 if (is_dir($dir)) {
		 // Ordnerinhalt auflisten und jedes Element nacheinander überprüfen
		 $dircontent=scandir($dir);
		 foreach ($dircontent as $c) {
		 // Wenn es sich um einen Ordner handelt, die Funktion rmdir(); aufrufen
		 if ($c!='.' and $c!='..' and is_dir($dir.'/'.$c)) {
			rmdir($dir.'/'.$c);
			// Wenn es eine Datei ist, diese löschen
		 } else if ($c!='.' and $c!='..') {
			unlink($dir.'/'.$c);
			}
		 }
		 // Den nun leeren Ordner löschen
		 rmdir($dir);
		 // Wenn es sich um eine Datei handelt, diese löschen
	 } else {
		unlink($dir);
	}
}

function erase_folder($dir)
    {
        $dir = realpath(trim($dir, '\\ /'));

        if ($dir && !is_writable($dir)) {
            chmod($dir, octdec(PMX_CHMOD_FULLUNOCK));
            clearstatcache(true, $dir);
        }

        if (!$dir || !$handle = opendir($dir)) {
            // gibts garnet
            return true;
        }

        /* Unterordner lesen */
        while (false !== ($file = readdir($handle))) {
            switch (true) {
                case $file == '.':
                case $file == '..':
                    break;
                case is_dir($file):
                    erase_folder($dir . '/' . $file);
                    break;
                default:
                    erase_file($dir . '/' . $file);
            }
        }
        closedir($handle);

        $ok = is_writable($dir) && @rmdir($dir);
        if (!$ok) {
            $err['dirs'][] = $dir;
        }
        return $ok;
    }

function erase_file($delfile)
    {

        // ###############################
        switch (true) {
            case !$delfile = realpath($delfile):
                // prima, die Datei gibts garnicht
                return true;
            case is_file($delfile):
                if (!is_writable($delfile)) {
                    chmod($delfile, octdec(PMX_CHMOD_FULLUNOCK));
                    clearstatcache(true, $delfile);
                }
                if (is_writable($delfile) && @unlink($delfile)) {
                    return true;
                }
                break;
            case is_dir($delfile):
                return erase_folder($delfile); /* rekursiver Aufruf für Subfolders */
            default:
                // keine Datei?
                $err['files'][] = $delfile;
                return false;
        }
        // zumindest versuchen zu leeren
        $ok = is_writable($delfile) && file_put_contents($delfile, '');
        if (!$ok) {
            $err['files'][] = $delfile;
			return $err;
        }
        return true;
    }

/**
 *  mx_unlink - löscht eine Datei
 *  
 *  @param [in] $file 
 *  @return 
 *  
 *  Date: 2020-02-22T17:51:09 
 *  created by : Olaf Herfurth 
 */
function mx_unlink($file)
{
    $old = getcwd(); // Save the current directory
	$path_to_file=dirname($file);

    unlink($file);
	
	if (is_file($file)) return @unlink($file); 
	return true;
}


// removes files and non-empty directories
function mx_rrmdir($dir) {
  if (is_dir($dir)) {
    $files = scandir($dir);
    foreach ($files as $file)
    if ($file != "." && $file != "..") mx_rrmdir("$dir/$file");
    rmdir($dir);
  }
  else if (file_exists($dir)) @unlink($dir);
}

// copies files and non-empty directories
function mx_rcopy($src, $dst) {
  if (file_exists($dst)) mx_rrmdir($dst);
  if (is_dir($src)) {
    @mkdir($dst);
    $files = scandir($src);
    foreach ($files as $file)
    if ($file != "." && $file != "..") mx_rcopy("$src/$file", "$dst/$file");
  }
  else if (file_exists($src)) @copy($src, $dst);
}

/* 
  check valide filename - not Foldername !!!
  
  @param [string] filename without path
  @return boolean 
  
*/

function mx_check_filename($filename =false)
{
	/* Fully portable filenames" lists these: A-Z a-z 0-9 . _ - */
	if (preg_match('/^[\w\-. ]+$/', $filename)) return true;
    return false;
}



function mx_download_file($url,$dest)
{
	// Initialize a file URL to the variable
  //$url = 'https://media.geeksforgeeks.org/wp-content/uploads/gfg-40.png';
  
  // Initialize the cURL session
  $ch = curl_init($url);
  
  // Initialize directory name where
  // file will be save
  
  $dir = $dest;
  
  // Use basename() function to return
  // the base name of file
  $file_name = basename($url);
  
  // Save file into file location
  $save_file_loc = $dir ;//. $file_name;
  
  // Open file
  $fp = fopen($save_file_loc, 'wb');
  
  // It set an option for a cURL transfer
  curl_setopt($ch, CURLOPT_FILE, $fp);
  curl_setopt($ch, CURLOPT_HEADER, 0);
  
  // Perform a cURL session
  curl_exec($ch);
  
  // Closes a cURL session and frees all resources
  curl_close($ch);
  
  // Close file
  fclose($fp);
  //file_put_contents($dir.".txt",$url);
}

?>
