<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 *
 * this file based on:
 * PHP-NUKE-Addon Newsletter 1.40
 * Copyright (c) by Andreas Guenther, 26.08.2001
 * webmaster@bariba.de / http://www.bariba.de
 * php-nuke Web Portal System - http://phpnuke.org/
 * Thatware - http://thatware.org/
 */

defined('mxMainFileLoaded') or die('access denied');

$module_name = basename(dirname(__FILE__));
mxGetLangfile($module_name);
require_once(PMX_MODULES_DIR . DS . $module_name . DS . 'includes' . DS . 'functions.php');
include(PMX_MODULES_DIR . DS . $module_name . DS . 'includes' . DS . 'settings.php');
pmxBase::set("plugins",0);
if (!MX_IS_USER) {
    $index = $nl_config['right_blocks'];
}

$pagetitle = _NL_TITLE;

/**
 * pmxNewsletter::main()
 * Prints the startscreen of the Newsletter
 *
 * @return
 */
function letter_main($data = array(), $message = '', $errors = array())
{
    global $prefix, $module_name, $nl_config;

    $status = '';
    $exist = array();
    if (MX_IS_USER) {
        $reqmail = (empty($data['email'])) ? '' : $data['email'];
        $userinfo = mxGetUserData();

        $qry = "SELECT *
                    FROM `{$prefix}_newsletter`
                    WHERE (uid=" . intval($userinfo['uid']) . "
                      OR email='" . mxAddSlashesForSQL($userinfo['email']) . "'
                      ) AND email<>'" . mxAddSlashesForSQL($reqmail) . "'
                    ORDER BY status DESC, id
                    LIMIT 1";
        $result = sql_query($qry);
        $row = sql_fetch_assoc($result);
        if (empty($row['uid'])) {
            $exist['email'] = $userinfo['email'];
        } else {
            $exist = $row;
            $exist['op'] = 'unsubscribe';
            if ($row['status'] == 2) {
                $status = _NL_SUBSCRIBED;
            }
        }
    }

    $defaults = array(/* Standardwerte */
        'email' => '',
        'type' => $nl_config['default_format'],
        'op' => 'subscribe',
        );

    $data = array_merge($defaults, $exist, $data);

    ob_start();

    if ($message) {

        ?><p class="important"><?php echo $message ?></p><br /><?php
    }

    if ($errors) {

        ?>
            <div class="warning error">
              <ul><li><?php echo implode('</li><li>', $errors) ?></li></ul>
            </div><br />
            <?php
    }

    if (!MX_IS_USER || !$message) {
        echo "<p>" . _NL_WELCOME . "</p>";

        if ($status) {

            ?><div class="note">
                <div style="background: transparent url(images/tu.gif) no-repeat scroll right center">
                <?php echo $status ?>
                </div>
              </div><?php
        }

        $captcha_object = load_class('Captcha', 'newsletteron');

        ?>

<form class="letter-form" method="post" action="modules.php?name=<?php echo $module_name ?>" method="post" class="letter-form">
  <input name="name" value="<?php echo $module_name ?>" type="hidden" />
  <?php echo _NL_EMAIL ?><br />
  <input type="text" maxlength="100" size="50" value="<?php echo $data['email'] ?>" name="email" placeholder="<?php echo _NL_YOURADDRESS ?>" /><br />
  <br />
  <?php echo _NL_CHOOSE ?><br />
  <select name="op">
    <option<?php echo (('subscribe' == $data['op']) ? ' selected="selected" class="current"' : '') ?> value="subscribe"><?php echo _NL_SUBSCRIBE ?></option>
    <option<?php echo (('unsubscribe' == $data['op']) ? ' selected="selected" class="current"' : '') ?> value="unsubscribe"><?php echo _NL_UNSUBSCRIBE ?></option>
  </select>
  <br />
  <br />
  <?php echo _NL_CHOOSETYPE ?><br />
  <select name="type">
    <option<?php echo ((0 == $data['type']) ? ' selected="selected" class="current"' : '') ?> value="0"><?php echo _NL_TYPETEXT ?></option>
    <option<?php echo ((1 == $data['type']) ? ' selected="selected" class="current"' : '') ?> value="1"><?php echo _NL_TYPEHTML ?></option>
  </select><br />
  <br />

  <?php if($captcha_object->get_active()){ ?>
  <div class="align-center">
    <?php echo $captcha_object->complete() ?>
  </div>
  <?php } //endif ?>

  <div align="center">
    <input type="submit" value="<?php echo _NL_SENDIT ?>" />
  </div>
</form>

<?php

    }

    $content = ob_get_clean();

    if (MX_IS_USER) {
        $content = '<div class="tabs-panel">' . $content . '</div>';
        require_once(PMX_SYSTEM_DIR . DS . 'mx_userfunctions.php');
        $view = new pmxUserPage($userinfo);
        $view->innertab = 'setnewsletter';
        $view->pagetitle = _NL_TITLE;
        $view->show($content);
    } else {
        $GLOBALS['pagetitle'] = _NL_TITLE;
        $GLOBALS['index'] = $nl_config['right_blocks'];

        include_once('header.php');
        title(_NL_TITLE);
        OpenTable();
        echo $content;
        CloseTable();
        include_once('footer.php');
    }
}

/**
 * pmxNewsletter::subscribe()
 * User wants to subscribe the letter
 *
 * @return
 */
function letter_subscribe()
{
    global $prefix, $sitename, $module_name, $slogan, $nl_config;

    $msg = '';
    $errors = array();

    if (MX_IS_USER) {
        $userinfo = mxGetUserData();
        $uid = $userinfo['uid'];
    } else {
        $uid = 0;
    }

    $data = array(/* Standardwerte */
        'email' => '',
        'type' => $nl_config['default_format'],
        );
    if ($_POST) {
        $_POST = mxStripSlashes($_POST);
        $data = array_merge($data, $_POST);
    }

    extract($data, EXTR_OVERWRITE);

    if (!mxCheckEmail($email) || $email == _NL_YOURADDRESS) {
        $errors[] = _NL_NOEMAIL;
    }

    if (!$errors) {
        /* Check, if subscribed already */
        $query = "SELECT id FROM {$prefix}_newsletter WHERE email='" . mxAddSlashesForSQL($email) . "'";
        $result = sql_query($query);
        list($id) = sql_fetch_row($result);
        if ($id) {
            $errors[] = _NL_ALREADY;
        }
    }

    $captcha_object = load_class('Captcha', 'newsletteron');
    if (!$captcha_object->check($data, 'captcha')) {
        if (isset($data['from_block']) && $data['from_block']) {
            // wenn der Request vom Block kommt und Captcha aktiviert ist,
            // einfach das Formular nochmal anzeigen
            return letter_main($data, '', $errors);
        } else {
            $errors[] = _CAPTCHAWRONG;
        }
    }

    if (!$errors) {
        $checkkey = mt_rand(100000, 999999); // immer 6 stellig :-)
        $reghost = gethostbyaddr(MX_REMOTE_ADDR);
        $query = "INSERT INTO {$prefix}_newsletter SET
                  uid = " . intval($uid) . ",
                  email = '" . mxAddSlashesForSQL($email) . "',
                  status = 1,
                  checkkey=" . $checkkey . ",
                  html=" . intval($type) . ",
                  regdate=" . MX_TIME . ",
                  regip='" . MX_REMOTE_ADDR . "',
                  reghost='" . mxAddSlashesForSQL($reghost) . "'";
        if (sql_query($query)) {
            $id = sql_insert_id();
            $buildlink = PMX_HOME_URL . '/modules.php?name=' . $module_name . '&confirm=' . $id . '.' . _letter_get_crc($email) . '.' . $checkkey;
            mxMail($email, _NL_SUBJECT, _NL_CONFTEXT . "\n" . $slogan . "\n\n" . $buildlink);
            $msg = _NL_SENDOK;
            $data['op'] = 'unsubscribe';
        } else {
            $errors[] = _CHANGESNOTOK;
        }
    }

    letter_garbage();

    return letter_main($data, $msg, $errors);
}

/**
 * pmxNewsletter::unsubscribe()
 * User wants to unsubscribe the letter
 *
 * @return
 */
function letter_unsubscribe()
{
    global $prefix, $sitename, $module_name, $slogan;

    $msg = '';
    $errors = false;

    $data = array(/* Standardwerte */
        'email' => '',
        'op' => 'unsubscribe',
        );

    if ($_POST) {
        $_POST = mxStripSlashes($_POST);
    }
    $data = array_merge($data, $_GET, $_POST);

    $id = 0;
    $control = 0;
    $captcha_object = null;
    switch (true) {
        case isset($_GET['unsubscribe']) && strpos($_GET['unsubscribe'], '.'):
            /* der unsubscribe-Link in den Newslettern */
            list($id, $control) = explode('.', $_GET['unsubscribe'] . '..');
            $query = "SELECT email FROM {$prefix}_newsletter WHERE id=" . intval($id);
            $result = sql_query($query);
            list($email) = sql_fetch_row($result);
            if (!$control || _letter_get_crc($email) != $control) {
                $errors[] = _NL_SUBNOT;
                return letter_main($data, $msg, $errors);
            }
            $data['email'] = $email;
            break;

        case empty($data['email']):
            $email = '';
            break;

        default:
            $captcha_object = load_class('Captcha', 'newsletteron');
            $email = $data['email'];
            /* Check, if subscribed */
            $query = "SELECT id FROM {$prefix}_newsletter WHERE email='" . mxAddSlashesForSQL($email) . "'";
            $result = sql_query($query);
            list($id) = sql_fetch_row($result);
            break;
    }

    $query = "DELETE FROM {$prefix}_newsletter WHERE id=" . intval($id);

    switch (true) {
        case !$email:
        case $email == _NL_YOURADDRESS:
            $errors[] = _NL_NOEMAIL;
            break;

        case $email && !$id:
            $msg = _NL_NOABO . ' (' . $email . ')';
            break;

        case ($captcha_object && !$captcha_object->check($data, 'captcha')) :
            $errors[] = _CAPTCHAWRONG;
            break;

        case !sql_query($query):
            $errors[] = _CHANGESNOTOK;
            break;

        default:
            $msg = _NL_UNSUBOK . ' (' . $email . ')';
            $data['op'] = 'subscribe';
            break;
    }

    return letter_main($data, $msg, $errors);
}

/**
 * pmxNewsletter::confirm()
 * Called by the confirmation link sets status=2 which means get the letter
 *
 * @return
 */
function letter_confirm()
{
    global $prefix, $module_name;

    $msg = '';
    $errors = false;
    $data = array(/* Standardwerte */
        'email' => '',
        'new_check' => 0,
        'confirm' => '',
        );

    $data = array_merge($data, $_GET);

    $control = 0;
    $id = 0;
    $checkkey = 0;

    $row = array();
    $checked = false;
    if (!empty($data['confirm']) && strpos($data['confirm'], '.')) {
        list($id, $control, $checkkey) = explode('.', $data['confirm'] . '..');

        switch (true) {
            case !$id:
            case !$control:
            case !$checkkey:
                /* Link falsch zusammengesetzt */
                $errors[] = _NL_SUBNOT . ' (1)';
                return letter_main($data, '', $errors);
        }

        $query = "SELECT id, email, checkkey FROM {$prefix}_newsletter WHERE id=" . intval($id);
        $result = sql_query($query);
        $row = sql_fetch_assoc($result);
        $checked = ($row['checkkey'] == $checkkey) && (_letter_get_crc($row['email']) == $control);
        if ($checked) {
            // Daten fürs Formular füllen
            $data['email'] = $row['email'];
        }
    } else if (!empty($data['email']) && !empty($data['new_check'])) {
        // beim alten Aufruf vor 1.12 wurden diese Daten direkt übergeben
        // alter Aufruf war:
        // &func=confirm&new_email=MAILADRESSE&new_check=DERCODE
        $query = "SELECT id
                      FROM {$prefix}_newsletter
                      WHERE (email='" . mxAddSlashesForSQL($data['email']) . "'
                        AND checkkey='" . mxAddSlashesForSQL($data['new_check']) . "')";
        $result = sql_query($query);
        list($id) = sql_fetch_row($result);
        $checked = !empty($id);
    }

    if (!$checked) {
        /* email nicht gefunden oder Bestätigungskey falsch */
        $errors[] = _NL_SUBNOT . ' (2)';
        return letter_main($data, '', $errors);
    }

    /* Bestätigungskey ok */
    $reghost = gethostbyaddr(MX_REMOTE_ADDR);
    $query = "UPDATE {$prefix}_newsletter SET
                    status = 2,
                    checkkey = 0,
                    regdate = " . MX_TIME . ",
                    regip = '" . MX_REMOTE_ADDR . "',
                    reghost = '" . mxAddSlashesForSQL($reghost) . "'
                  WHERE id = " . intval($id);

    if (!sql_query($query)) {
        $errors[] = _CHANGESNOTOK;
        return letter_main($data, '', $errors);
    }
    return letter_main($data, _NL_SUBOK);
}

/**
 * Calls the different functions of the Addon
 * -------------------------------------------------------------------------------
 * Ruft die einzelnen Userfunktionen fuer das Addon auf
 */

$func = (empty($_GET['func'])) ? '' : $_GET['func'];
$op = (empty($_POST['op'])) ? '' : $_POST['op'];

switch (true) {
    case 'subscribe' == $op: // per Formular
        return letter_subscribe();
        break;

    case 'unsubscribe' == $op: // per Formular
    case isset($_GET['unsubscribe']): // per Link im Newsletter
    // alter Aufruf vor pragmaMx 1.12
    case 'action' == $func && (isset($_GET['new_sub']) && 'unsub' == $_GET['new_sub']):
        return letter_unsubscribe();
        break;

    case isset($_GET['confirm']): // per Link im Bestätigungsmail
    // alter Aufruf vor pragmaMx 1.12
    case 'confirm' == $func && isset($_GET['new_check']):
        return letter_confirm();
        break;

    default:
        return letter_main();
        break;
}

?>