<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 *
 *
 * Part of phpBB integration
 * Copyright (c) 2001 by
 * Richard Tirtadji AKA King Richard (rtirtadji@hotmail.com)
 * Hutdik Hermawan AKA hotFix (hutdik76@hotmail.com)
 * http://www.phpnuke.web.id
 */

defined('mxMainFileLoaded') or die('access denied');

global $url_smiles, $smilie;

function pmPrepDisplay($msg)
{
    return msg_smile(msg_bbencode(make_clickable(mxNL2BR($msg))));
}

function pmUndoPrepDisplay($msg)
{
    return msg_bbdecode(msg_desmile(mxUndoNltobr($msg)));
}

/*
 * Changes :) to an <IMG> tag based on the smiles table in the database.
 * Smilies must be either:
 *   - at the start of the message.
 *   - at the start of a line.
 *   - preceded by a space or a period.
 * This keeps them from breaking HTML code and BBCode.
 * TODO: Get rid of global variables.
 */
function msg_smile($message)
{
    global $url_smiles, $smilie;
    if (empty($url_smiles) || empty($smilie)) {
        include("modules/" . basename(dirname(__FILE__)) . "/config.php");
    }
    reset($smilie);
    $message = " " . $message; // Pad it with a space so the regexp can match.
    $message = str_replace(array('&#41;', '&#124;'), array(')', '|'), $message); /// macht mxSecureValue() rückgängig
    foreach ($smilie as $vals) {
        $smile_code = preg_quote($vals[0]);
        $smile_code = str_replace('/', '//', $smile_code);
        $message = preg_replace("/([\n\\ \\.])$smile_code/si", '\1<img src="' . $url_smiles . '/' . $vals[2] . '">', $message);
    }
    // Remove padding, return the new string.
    $message = trim($message);
    return $message;
}

/*
 * Changes a Smiliy <img> tag into its corresponding smile
 * TODO: Get rid of golbal variables, and implement a method of distinguishing between :D and :grin: using the <img> tag
 */
function msg_desmile($message)
{
    global $url_smiles, $smilie;
    if (empty($url_smiles) || empty($smilie)) {
        include("modules/" . basename(dirname(__FILE__)) . "/config.php");
    }
    reset($smilie);
    $message = str_replace(array('&#41;', '&#124;'), array(')', '|'), $message); /// macht mxSecureValue() rückgängig
    foreach ($smilie as $vals) {
        $smile_code = preg_quote($vals[0]);
        $smile_code = str_replace('/', '//', $smile_code);
        $message = str_replace(" <img src=\"$url_smiles/" . $vals[2] . "\"> ", $smile_code, $message);
    }
    return $message;
}

function msg_bbencode($message)
{
    global $prefix;
    // [CODE] and [/CODE] for posting code (HTML, PHP, C etc etc) in your posts.
    $message = str_replace(array('&#41;', '&#124;'), array(')', '|'), $message); /// macht mxSecureValue() rückgängig
    $matchCount = preg_match_all("#\[code\](.*?)\[/code\]#si", $message, $matches);
    for ($i = 0; $i < $matchCount; $i++) {
        $currMatchTextBefore = preg_quote($matches[1][$i]);
        $currMatchTextAfter = htmlspecialchars($matches[1][$i]);
        $message = preg_replace("#\[code\]$currMatchTextBefore\[/code\]#si", "<!-- BBCode Start --><table align=\"center\" width=\"85%\"><tr><td><font class=\"tiny\">Code:</font><hr size=1 noshade></td></tr><tr><td><font class=\"tiny\"><pre>$currMatchTextAfter</pre></font></td></tr><tr><td><hr size=1 noshade></td></tr></table><!-- BBCode End -->", $message);
    }
    // [QUOTE] and [/QUOTE] for posting replies with quote, or just for quoting stuff.
    $i = 0;
    while (preg_match("#\[quote\](.*)\[/quote\]#si", $message, $matches) && $i < 20) {
        $message = str_replace($matches[0], pmGetQuoteHtml('start') . $matches[1] . pmGetQuoteHtml('end'), $message);
        $i++;
    }
    // [b] and [/b] for bolding text.
    $message = preg_replace("#\[b\](.*?)\[/b\]#si", "<!-- BBCode Start --><b>\\1</b><!-- BBCode End -->", $message);
    // [i] and [/i] for italicizing text.
    $message = preg_replace("#\[i\](.*?)\[/i\]#si", "<!-- BBCode Start --><i>\\1</i><!-- BBCode End -->", $message);
    // [url]www.phpbb.com[/url] code..
    $message = preg_replace("#\[url\](http://)?(.*?)\[/url\]#si", "<!-- BBCode Start --><a href=\"http://\\2\" target=\"_blank\">\\2</a><!-- BBCode End -->", $message);
    // [url=www.phpbb.com]phpBB[/url] code..
    $message = preg_replace("#\[url=(http://)?(.*?)\](.*?)\[/url\]#si", "<!-- BBCode Start --><a href=\"http://\\2\" target=\"_blank\">\\3</a><!-- BBCode End -->", $message);
    // [email]user@domain.tld[/email] code..
    $message = preg_replace("#\[email\](.*?)\[/email\]#si", "<!-- BBCode Start --><a href=\"mailto:\\1\">\\1</a><!-- BBCode End -->", $message);
    // [img]image_url_here[/img] code..
    $message = preg_replace("#\[img\](.*?\.(png|jpe?g|gif))[[:space:]]*?\[/img\]#si", "<!-- BBCode Start --><img src=\"\\1\"><!-- BBCode End -->", $message);
    // unordered list code..
    $matchCount = preg_match_all("#\[list\](.*?)\[/list\]#si", $message, $matches);
    for ($i = 0; $i < $matchCount; $i++) {
        $currMatchTextBefore = preg_quote($matches[1][$i]);
        $currMatchTextAfter = preg_replace("#\[\*\]#si", "<li>", $matches[1][$i]);
        $message = preg_replace("#\[list\]$currMatchTextBefore\[/list\]#si", "<!-- BBCode ulist Start --><ul>$currMatchTextAfter</ul><!-- BBCode ulist End -->", $message);
    }
    // ordered list code..
    $matchCount = preg_match_all("#\[list=([a1])\](.*?)\[/list\]#si", $message, $matches);
    for ($i = 0; $i < $matchCount; $i++) {
        $currMatchTextBefore = preg_quote($matches[2][$i]);
        $currMatchTextAfter = preg_replace("#\[\*\]#si", "<li>", $matches[2][$i]);
        $message = preg_replace("#\[list=([a1])\]$currMatchTextBefore\[/list\]#si", "<!-- BBCode olist Start --><ol type=\\1>$currMatchTextAfter</ol><!-- BBCode olist End -->", $message);
    }
    return $message;
}

function msg_bbdecode($message)
{
    $message = str_replace(array('&#41;', '&#124;'), array(')', '|'), $message); /// macht mxSecureValue() rückgängig

    $message = str_replace(pmGetQuoteHtml('start'), "[quote]", $message);
    $message = str_replace(pmGetQuoteHtml('end'), "[/quote]", $message);
    // Undo [b] and [i]
    $message = preg_replace("#<!-- BBCode Start --><b>(.*?)</b><!-- BBCode End -->#s", "[b]\\1[/b]", $message);
    $message = preg_replace("#<!-- BBCode Start --><i>(.*?)</i><!-- BBCode End -->#s", "[i]\\1[/i]", $message);
    // Undo [url] (long form)
    $message = preg_replace("#<!-- BBCode u2 Start --><a href=\"([a-z]+?://)(.*?)\" target=\"_blank\">(.*?)</a><!-- BBCode u2 End -->#s", "[url=\\1\\2]\\3[/url]", $message);
    // Undo [url] (short form)
    $message = preg_replace("#<!-- BBCode u1 Start --><a href=\"([a-z]+?://)(.*?)\" target=\"_blank\">(.*?)</a><!-- BBCode u1 End -->#s", "[url]\\3[/url]", $message);
    // Undo [email]
    $message = preg_replace("#<!-- BBCode Start --><a href=\"mailto:(.*?)\">(.*?)</a><!-- BBCode End -->#s", "[email]\\1[/email]", $message);
    // Undo [img]
    $message = preg_replace("#<!-- BBCode Start --><img src=\"(.*?)\" border=\"0\"><!-- BBCode End -->#s", "[img]\\1[/img]", $message);
    // Undo lists (unordered/ordered)
    // <li> tags:
    $message = str_replace("<!-- BBCode --><li>", "[*]", $message);
    // [list] tags:
    $message = str_replace("<!-- BBCode ulist Start --><ul>", "[list]", $message);
    // [list=x] tags:
    $message = preg_replace("#<!-- BBCode olist Start --><ol type=([A1])>#si", "[list=\\1]", $message);
    // [/list] tags:
    $message = str_replace("</ul><!-- BBCode ulist End -->", "[/list]", $message);
    $message = str_replace("</ol><!-- BBCode olist End -->", "[/list]", $message);
    return $message;
}

if (!function_exists('pmGetQuoteHtml')) {
    function pmGetQuoteHtml($part)
    {
        switch ($part) {
            case 'start':
                return '<!-- bbcode quote start --><div class="quote"><div class="quote-header">' . _TEXTQUOTE . ':</div>';
            case 'end':
                return '</div><!-- bbcode quote end -->';
        }
    }
}

?>
