<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 584 $
 * $Author: PragmaMx $
 * $Date: 2022-08-29 21:51:21 +0200 (Mo, 29. Aug 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');
pmxBase::set("plugins",0);
$plugins=0;
/**
 * pmx_search
 *
 * @package pragmaMx
 * @author tora60
 * @copyright Copyright (c) 2014
 * @version $Id: index.php 584 2022-08-29 19:51:21Z PragmaMx $
 * @access public
 */
class pmx_search {
    protected $_modulename;

    /* alle Daten, die aus $_REQUEST gebildet werden und gleichzeitig deren Standardwerte */
    protected $req = array(
        /* Der eigentliche Suchparameter aus Request */
        'query' => '',
        /* Der Suchparameter für mySql */
        'sqlstring' => '',
        /* Der Suchparameter zur Weitergabe in URL's */
        'urlstring' => '',
        /* Der Suchparameter zur Ausgabe auf der Webseite */
        'htmlstring' => '',
        'modlist' => array(),
        /* Anzahl der Ergebnisse pro Seite */
        'offset' => 12,
        'sort_order' => 'DESC',
        'sort_field' => 'date',
        );

    /* Das Suchergebnis */
    protected $items = array();

    /* Template */
    protected $_template;

    /* maximale Anzahl der Ergebnisse pro Modul */
    protected $_permod = 50;

    /**
     * pmx_search::__construct()
     */
    public function __construct ()
    {
        $this->_modulename = basename(__DIR__);
        mxGetLangfile($this->_modulename);

        $this->_handle_request();

        $this->items = $this->getitems();

        $this->_template = load_class('Template');
        $this->_template->init_path(__DIR__);
        $this->_template->assign('modulename', $this->_modulename);

        return $this->main();
    }

    protected function _handle_request()
    {
        /* ab hier den Suchstring zusammenstellen */
        switch (true) {
            case !empty($_REQUEST['q']):
                $request['query'] = $_REQUEST['q'];
                unset($request['q']);
                break;
            case !empty($_REQUEST['query']):
                $request['query'] = $_REQUEST['query'];
                break;
            default:
        }

        if (isset($_REQUEST['offset']) && intval($_REQUEST['offset'])) {
            $request['offset'] = intval($_REQUEST['offset']);
        }

        /* Suche auf bestimmte Module einschränken */
        switch (true) {
            case !isset($_REQUEST['m']):
                break;
            case is_array($_REQUEST['m']):
                $tmplist = $this->_get_allmodules();
                /* wenn das Array leer bleibt, dann werden ja automatisch wieder alle Module abgefragt ;-) */
                foreach ($_REQUEST['m'] as $key) {
                    if (array_key_exists($key, $tmplist)) {
                        $request['modlist'][] = $key;
                    }
                }
                break;
            case is_string($_REQUEST['m']):
                $tmplist = $this->_get_allmodules();
                if (array_key_exists($_REQUEST['m'], $tmplist)) {
                    $request['modlist'] = array($_REQUEST['m']);
                }
                break;
        }

        /* ############################# */
        /* TODO: folgendes ist nur zum Testen, das muss dann über $_GET geregelt werden */
        // $_REQUEST['sort_order'] = 'ASC';
        $_REQUEST['sort_order'] = 'DESC';
        // $_REQUEST['sort_field'] = 'relevance';
        // $_REQUEST['sort_field'] = 'title';
        $_REQUEST['sort_field'] = 'date';
        /* ############################# */

        if (isset($_REQUEST['sort_order'])) {
            $request['sort_order'] = $_REQUEST['sort_order'];
        }

        if (isset($_REQUEST['sort_field'])) {
            $request['sort_field'] = $_REQUEST['sort_field'];
        }

        $request = array_merge($this->req, $request);

        if ($request['query']) {
            /* wird überhaupt was gefragt? Wenn Ja, dann gleich die slashes weg. */
            $request['query'] = trim(stripslashes($request['query']));

            /* Der Suchparameter für mySql */
            $request['sqlstring'] = $this->convert_query($request['query']);

            /* Der Suchparameter zur Weitergabe in URL's */
            $request['urlstring'] = urlencode($request['sqlstring']);

            /* Der Suchparameter zur Ausgabe auf der Webseite */
            $request['htmlstring'] = htmlspecialchars($request['sqlstring']);
        }

        $this->req = $request;
        // mxDebugFuncVars($this->req);
    }

    /**
     * pmx_search::getitems()
     *
     * @return
     */
    protected function getitems()
    {
        global $prefix;

        /* Suchergebnis initialisieren */
        $items = array();

        /* Zum sicherstellen, dass immer alle Werte gesetzt sind */
        $defaultentry = array(
            'date' => time(), // das Datum des Beitrags als unixtimestamp/integer
            'module' => '', // der Modulname
            'moduletitle' => '', // der Modultitel
            'path' => array(), // Pfad, z.B. über Modul > Kategorie > Beitrag
            'title' => _NOTITLE, // Titel des Beitrags
            'text' => '', // der gesamte Text des Beitrags
            'link' => '', // url/Link zum Beitrag
            'views' => 0, // Beitrag wie oft gesehen?
            'relevance' => 0, // Relevanz nach der mySql Volltextsuche
            );

        /* Zum sicherstellen, dass $sort_field einen korrekten Wert hat */
        if (empty($this->sort_field) || !isset($defaultentry[$this->sort_field])) {
            $this->sort_field = 'date';
        }

        /* Cache initialisieren */
        $cacheid = $this->sqlstring . $this->offset . $this->sort_field . $this->sort_order . implode('|', $this->modlist) . $this->_get_cacheid();
        $cache = load_class('Cache');

        /* Ist was im Cache? */
        if (($items = $cache->read($cacheid)) === false) {
            // Nö, nix drin...
			$items=array();
            $parameters = array(/*  */
                "type" => 'items',
                "search" => sql_real_escape_string($this->sqlstring),
                // TODO: diese Anzahl nicht verwenden, wenn nur 1 Modul abgefragt wird
                "limit" => intval($this->_permod),
                // TODO: diese Optionen in den Hook Dateien berücksichtigen
                "sort_field" => $this->sort_field,
                "sort_order" => $this->sort_order,
                'only_allowed' => true,
                'only_active' => true,
                /* Suche auf bestimmte Module einschränken */
                'modlist' => $this->modlist,
                );

            $hook = load_class('Hook', 'search.content');
            $hook->set($parameters);

            $searchentries = $hook->get(array());
            if ($searchentries && is_array($searchentries)) {

            /* Auch alle Module durchsuchen */
            $modules = sql_query("SELECT `title`, `custom_title`
            FROM {$prefix}_modules
            WHERE active=1
              AND  MATCH( `title`, `custom_title`)
                AGAINST('" . sql_real_escape_string($this->sqlstring) . "' IN BOOLEAN MODE)");
            while (list($modulename, $custom_title) = sql_fetch_row($modules)) {
                if (mxModuleAllowed($modulename)) {
                    array_unshift($searchentries, array(/* Moduldaten */
                            'module' => $modulename, // der Modulname
                            'moduletitle' => $custom_title, // der Modultitel
                            'title' => $custom_title, // Titel des Beitrags
                            'link' => 'modules.php?name=' . $modulename, // url/Link zum Beitrag
                            ));
                }
            }

            $i = 0;

                foreach ($searchentries as $entry) {
                    $i++;
                    $entry = array_merge($defaultentry, $entry);
                    $id = $entry[$this->sort_field] . sprintf('.%06d' , $i);

                    $entry['title'] = ($entry['title']) ? $this->prepare_text($entry['title'], 80) : _NOTITLE;
                    $entry['text'] = ($entry['text']) ? $this->prepare_text($entry['text'], 200) : '';

                    if ($entry['path']) {
                        foreach ($entry['path'] as $key => $title) {
                            $title = $this->prepare_text($title, 200);
                            if ($title) {
                                $entry['path'][$key] = $title;
                            } else {
                                unset($entry['path'][$key]);
                            }
                        }
                    }

                    /* 1.path, am Anfang der Link zum Modul */
                    $path = array("modules.php?name=$entry[module]" => $entry['moduletitle']);
                    /* 2.path, zwischendrin der evtl. vom Modul ermittelte Pfad */
                    $path += $entry['path'];
                    /* 3.path, zum Schluss der Link zu diesem Eintrag */
                    $path[$entry['link']] = $entry['title'];
                    $entry['path'] = $path;
                    // mxDebugFuncVars($entry);
                    $items[$id] = $entry;
                }
            }

            /* Cache schreiben */
            $cache->write($items, $cacheid, 18000); // 5 Stunden Cachezeit
        }

        /* das Array in der gewünschten Reihenfolge sortieren */
        switch (true) {
            case !$items:
                break;
            case $this->sort_order == 'ASC':
                ksort($items);
                break;
            default:
                krsort($items);
        }

        return $items;
    }

    /**
     * pmx_search::main()
     *
     * @return
     */
    protected function main()
    {
        pmxHeader::add_style(PMX_MODULES_PATH . $this->_modulename . '/style/style.css');

        $pagetitle = _SEARCH_MODUL;
        $this->_template->assign('mask', $this->searchmask());
        $this->_template->assign('results', $this->searchresults());
        $this->_template->assign('more', $this->searchmore());

        include('header.php');
        $this->_template->display('main.html');
        include('footer.php');
    }

    /**
     * pmx_search::searchmask()
     *
     * @return
     */
    protected function searchmask()
    {
        $file = $this->_template->find_language_template(pmxBase::currentlang(), 'help.');
        if ($file) {
            $this->_template->add_path(dirname($file));
            $help = $this->_template->fetch(basename($file));
        } else {
            $help = _SEARCH_NOHELP;
        }

        $this->_template->assign('query', $this->htmlstring);
        $this->_template->assign('allmodules', $this->_get_allmodules());
        $this->_template->assign('modlist', $this->modlist);
        $this->_template->assign('help', $help);
        // return $this->_template->fetch('searchmask-tabs.html');
        return $this->_template->fetch('searchmask.html');
    }

    /**
     * pmx_search::searchresults()
     *
     * @return
     */
    protected function searchresults()
    {
        switch (true) {
            case !$this->query:
                return;
            case !$this->items:
                $message = _NOMATCHES;
                $pagination = '';
                break;
            default:
                $message = '';
                $start = (empty($_GET['start']) || $_GET['start'] < 0) ? 0 : intval($_GET['start']);
                $show = (empty($_GET['show']) || $_GET['show'] < 0) ? intval($this->offset) : intval($_GET['show']);

                $allrows = count($this->items);
                if ($allrows > $show) {
                    // Ausschnitt extrahieren
                    $this->items = array_slice($this->items, $start, $show, true);
                }

                $link = 'modules.php?name=' . $this->_modulename . '&amp;q=' . preg_replace('#%+#', '%%', $this->urlstring) . '&amp;start=%d';

                if (isset($_GET['show'])) {
                    $link .= '&amp;show=' . $show;
                }

                $pagination = $this->pagination($allrows, $start, $show, $link);
        }

        $this->_template->assign('items', $this->items);
        $this->_template->assign('message', $message);
        $this->_template->assign('pagination', $pagination);

        return $this->_template->fetch('results.html');
    }

    /**
     * pmx_search::searchmore()
     *
     * @return
     */
    protected function searchmore()
    {
        if ($this->query && $this->query != '*' && $this->query != '%') {
            $this->_template->assign('search_htmlstring', $this->htmlstring);
            $this->_template->assign('search_urlstring', $this->urlstring);

            return $this->_template->fetch('searchmore.html');
        }
    }

    /**
     * pmx_search::convert_query()
     * INFO: http://dev.mysql.com/doc/refman/5.1/de/fulltext-boolean.html
     *
     * @param mixed $query
     * @return
     */
    protected function convert_query($query)
    {
        /* Slashes und umgewandelte HTML-Tags von mxSecureValue() zurücknehmen */
        $out = str_replace(array ('&quot;', '&lt;', '&gt;'), array ('"', '<', '>'), $query);
        /* alle HTML-Tags entfernen */
        $out = strip_tags($out);

        /* Hinweis: [+-><()~*"]  = Operatoren zur mySql-Volltextsuche */
        /* http://dev.mysql.com/doc/refman/5.1/de/fulltext-boolean.html */

        /* Leerzeichen um bestimmte Volltextsuch Operatoren entfernen */
        $out = preg_replace('#\s+(\*)#', '\1', $out); // vor * Operator
        $out = preg_replace('#([+-><~])\s+#', '\1', $out); // nach sonstigem Operator

        /* raus damit */
        return $out;
    }

    /**
     * pmx_search::convert_query2marks()
     * INFO: http://dev.mysql.com/doc/refman/5.1/de/fulltext-boolean.html
     *
     * @return array ()
     */
    protected function convert_query2marks()
    {
        /* passiert ja nur einmal */
        static $out;
        if (isset($out)) {
            return $out;
        }

        /* ab hier jetzt die Suchfolge aufbereiten um die Fundstellen im Text zu markieren */
        $query = $this->sqlstring;
        /* Array für die aufgesplittete Suchfolge */
        $strings = array();

        /* Hinweis: [+-><()~*"]  = Sonderzeichen zur mySql-Volltextsuche */

        /* Strings in Anführungszeichen eingeschlosssen suchen, findet z.B. '+"ein Test"' */
        if (preg_match_all('#([+-><()~ ]*)"([^"]+)"#', $query, $matches)) {
            foreach ($matches[1] as $key => $pre) {
                /* Strings mit Vorzeichen "-" ignorieren */
                if ('-' != $pre) {
                    $query = trim(str_replace($matches[0][$key], '', $query));
                    $strings[] = $matches[2][$key];
                }
            }
        }

        /* Strings mit Minus Vorzeichen "-" ausschneiden, die woll ma net
         * Sternchen '*' bleiben aber erhalten */
        $query = trim(preg_replace('#-[^ ]+\*?#', '', $query));

        /* sonstige Volltextsuch Sonderzeichen und Leerzeichen als Trennzeichen verwenden */
        $tmp = preg_split('#[+-><()~ "]+#', $query);
        if ($tmp) {
            $strings = array_merge($strings, $tmp);
        }
        $strings = array_filter(array_unique($strings));

        /* die aufgesplittete Suchfolge als regex */
        $pattern = '#(^|\W)(' . str_replace('\|', '|', preg_quote(implode('|', $strings), '#')) . ')(\W|$)#iu';
        /* Behandlung des Sternchens * korrigieren */
        $pattern = str_replace('\*', '\w*', $pattern);
        /* Leerzeichen korrigieren, weil im Text immer 2 Leerzeichen
         * aufeinander folgen, siehe Funktion prepare_text() */
        $pattern = str_replace(' ', '\s+', $pattern);

        /* Anzahl der Einzelteile der Suchfolge */
        $count = count($strings);

        $out = compact('strings', 'pattern', 'count');

        return $out;
    }

    /**
     * pmx_search::prepare_text()
     *
     * @param mixed $text
     * @param mixed $len
     * @return
     */
    protected function prepare_text($text, $len = 200)
    {
        if (!$text) {
            return '';
        }

        /* sicherstellen dass um jeden Tag ein Leerzeichen ist, wegen <br/> */
        $text = str_replace(array('<', '>'), array(' <', '> '), $text);

        /* html-Tags entfernen */

        $text = strip_tags($text,"<img><span>");

        /* ein oder mehrere Leerzeichen durch 2 Leerzeichen ersetzen, damit weiter unten die Suche matched */
        $text = str_replace(array('&nbsp;', '  '), ' ', $text);
        $text = preg_replace('#\s+#u', '  ', $text);
        /* den Text in Einzelteile bestimmter Länge teilen, aber
         * nur an Leerzeichen trennen. Der Teilstring, der später am
         * meisten die Suchfolge enthält wird verwendet */
        $text = wordwrap($text, $len , '<<-->>', false);
        $parts = explode('<<-->>', $text);

        /* Array für die aufgesplittete Suchfolge */
        // /* die aufgesplittete Suchfolge als regex */
        // /* Anzahl der Einzelteile der Suchfolge */
        $marks = $this->convert_query2marks();

        /* maximale Anzahl in der Schleife durch die Teilstrings des Textes */
        $count_max = 0;
        /* standarmässig erstmal den ersten Teilstring des Textes verwenden */
        $text = $parts[0];
        /* der ausgewählte Teilstring für die Punktierung */
        $index = 0;
        /* hier guggen, ob und wie oft in einem der Teilstrings die
         * Suchfolge vorkommt, dann evtl. diesen Teilstring verwenden */
        foreach ($parts as $key => $value) {
   /* Alle relevanten Tags aus dem zu ändernden Inhalt auslesen */
    // Tags in der Form <tag>text</tagende> z.B. <a>irgendwas</a>
    preg_match_all('#<(a|form|font|iframe|object|title|meta|script|noscript|code|header|footer|adress|label|table|thead|h1|h2|h3|h4|h5|h6)[^>]+>.+</\1>#isU', $value, $matches1);
    // Tags in der Form <tag />, z.B. <img />
    preg_match_all('#<(img|button)[^>]+>#isU', $value, $matches2);
    // preg_match_all("#<(h[1-9])[^>]+>(.*)</[^>]+>#U", $value, $matches3);
    $matches = array_merge($matches1[0], $matches2[0]);
	
    $temp = array();
    foreach ($matches as $value) {
        $key = '~|~' . base64_encode($value) . '~|~';
        /* Die Platzhalter als Schlüssel, der Originalwert als Wert */
        $temp[$key] = $value;
    }

    /* Die Tags durch einen eindeutigen Platzhalter ersetzen */
    if ($temp) {
        $value = str_replace(array_values($temp), array_keys($temp), $value);
    }
            /* markiere Fundstellen */
            $tmptext = preg_replace($marks['pattern'], '\1<b>\2</b>\3', $value, -1, $count_found);
            if ($count_found > $count_max) {
                $count_max = $count_found;
                $text = $tmptext;
                $index = $key;
            }
            if ($count_found >= $marks['count']) {
                break;
            }
        }

        /* Pünktierung um den String, wenn nicht das Ende oder Anfang vom Gesamttext... */
        $tmp = count($parts);
        switch (true) {
            case $tmp == 1:
                // keine Punkte
                break;
            case $index < $tmp && $index > 0:
                // punkte vor und danach
                $text = '...&nbsp;' . trim($text) . '&nbsp;...';
                break;
            case $index > 0:
                // punkte davor
                $text = '...&nbsp;' . trim($text);
                break;
            case $index < $tmp:
                // punkte danach
                $text = trim($text) . '&nbsp;...';
                break;
        }

        return $text;
    }

    /**
     * pmx_search::_get_allmodules()
     *
     * Ermittelt die Liste aller zur Suche verfügbaren Module
     *
     * @return
     */
    protected function _get_allmodules()
    {
        $cacheid = __METHOD__ . $this->_get_cacheid();
        $cache = load_class('Cache');

        /* Ist was im Cache? */
        if (($items = $cache->read($cacheid)) === false) {
            $hook = load_class('Hook', 'search.content');
            $hook->type = 'modules';

            $items = $hook->get(array());
            if (!$items || !is_array($items)) {
                $items = array();
            }

            /* Cache schreiben */
            $cache->write($items, $cacheid, 18000); // 5 Stunden Cachezeit

        }

        return $items;
    }

    /**
     * pmx_search::_get_cacheid()
     *
     * eine für den User nzw. dessen Gruppen, spezifische ID als Zusatz
     * zur Cache-ID, damit only_allowed nicht durcheinander kommt
     *
     * @return
     */
    protected function _get_cacheid()
    {
        if (MX_IS_USER) {
            $id = mxGetUserData();
            $id = implode('|', $id['groups']);
        } else {
            $id = '';
        }
        return $id;
    }

    /**
     * pmx_search::pagination()
     * Page Numbering
     *
     * @param mixed $count_rows
     * @param mixed $start
     * @param mixed $show
     * @param mixed $link
     * @return
     */
    protected function pagination($count_rows, $start, $show, $link)
    {
        $pages = ceil($count_rows / $show);
        // wenn weniger als 2 Seiten, nix anzeigen
        if ($pages <= 1) {
            return;
        }

        $max = intval($start + $show);
        $prev = intval($start - $show);
        $currentpage = ceil($max / $show);
        $start = '';
        $ende = '';
        $counter = 1;
        $part = array();

        if ($currentpage > 3 && $pages > 6) {
            $start = '<a href="' . sprintf($link, 0) . '" title="' . _GOTOPAGEFIRST . '">1&nbsp;<span class="arrows">&laquo;</span></a><span class="points">..</span>';
            if ($currentpage >= $pages - 3) {
                $counter = $currentpage - ($currentpage - $pages + 4);
            } else {
                $counter = $currentpage - 1;
            }
        }
        // Schleife durch Suchergebnisse
        while ($counter <= $pages) {
            $mintemp = ($show * $counter) - $show;
            switch (true) {
                case ($counter > 5) && ($counter > $currentpage + 2) && ($counter < $pages) && $pages > 6:
                    $ende = '<span class="points">..</span><a href="' . sprintf($link, ($pages-1) * $show) . '" title="' . _GOTOPAGELAST . '"><span class="arrows">&raquo;</span>&nbsp;' . $pages . ' </a>';
                    break 2;
                case $counter == $currentpage:
                    $part[] = '<a href="' . sprintf($link, $mintemp) . '" title="' . sprintf(_PAGEOFPAGES, $currentpage, $pages) . '" class="current">' . $counter . '</a>';
                    break;
                default:
                    $part[] = '<a href="' . sprintf($link, $mintemp) . '" title="' . _GOTOPAGE . ' ' . $counter . '">' . $counter . '</a>';
            }

            $counter++;
        }

        if ($part) {
            return '
            <div class="pagination align-right">
              <span class="counter">' . sprintf(_PAGEOFPAGES, $currentpage, $pages) . '</span>
              ' . $start . implode('', $part) . $ende . '
            </div>';
        }
    }

    /**
     * pmxBackend::__get()
     *
     * @param mixed $name
     * @return
     */
    public function __get($name)
    {
        if (array_key_exists($name, $this->req)) {
            return $this->req[$name];
        }
        $trace = debug_backtrace();
        trigger_error('undefined property \'' . $name . '\' in ' . mx_strip_sysdirs($trace[0]['file']) . ' line ' . $trace[0]['line'], E_USER_NOTICE);
        return null;
    }
}

$tmp = new pmx_search();
$tmp = null;

?>