<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 *
 * Based on an earlier Idea for phpNuke 4.4 from Jeff Lambert (jeffx@ican.net)
 */

defined('mxMainFileLoaded') or die('access denied');

require_once(dirname(dirname(__FILE__)) . DS . 'includes' . DS . 'class.siteupdate.php');

/**
 * pmxSiteup_admin
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2011
 * @version $Id: admin.php 499 2022-05-11 12:15:10Z PragmaMx $
 * @access public
 */
class pmxSiteup_admin extends pmxSiteup {
    private $_defaults = array();

    /**
     * pmxSiteup_admin::__construct()
     */
    public function __construct($op)
    {
        parent::__construct();

        if (!$this->_is_admin) {
            return mxErrorScreen(_ACCESSDENIED);
        }

        $admindata = mxGetAdminSession();
        // mxSessionSetVar('panel', MX_ADMINPANEL_CONTENT);
        $this->_defaults = array(/* Standardwerte */
            'id' => 0,
            'aid' => $admindata['aid'],
            'date' => time(),
            'files' => '',
            'updates' => '',
            'bugs' => '',
            'save' => false,
            'preview' => false,
            );

        switch ($op) {
            case $this->_module_name . '/add':
                $this->add();
                break;

            case $this->_module_name . '/add/send':
                $this->add_send();
                break;

            case $this->_module_name . '/del':
                $this->del();
                break;

            case $this->_module_name . '/del/send':
                $this->del_send();
                break;

            case $this->_module_name . '/edit':
                $this->edit();
                break;

            case $this->_module_name . '/edit/send':
                $this->edit_send();
                break;

            default:
                $this->main();
                break;
        }
    }

    /**
     * pmxSiteup_admin::main()
     *
     * @return
     */
    function main()
    {
        $content = $this->list_entries();
        $this->_tabs[_SITEUP_ENTRIES] = '';
        return $this->adminpage($content);
    }

    /**
     * pmxSiteup_admin::adminpage()
     *
     * @param mixed $content
     * @return
     */
    function adminpage($content)
    {
        // TODO: !!!!!!!!!!!!!!!
        $GLOBALS['pagetitle'] = _SITEUP_MODULENAME;

        $this->_template->assign('content', $content);
        $this->_template->assign('messages', $this->_messages);
        $this->_template->assign('errors', $this->_errors);
        $this->_template->assign('tabs', $this->_tabs);
        $this->_template->assign('module_name', $this->_module_name);

        include_once('header.php');
        GraphicAdmin();
        $this->_template->display('admin.page.html');
        include_once('footer.php');
    }

    /**
     * pmxSiteup_admin::preview()
     *
     * @param mixed $data
     * @param mixed $caption
     * @return
     */
    function preview($data, $caption = false)
    {
        $this->_template->assign('caption', $caption);
        $this->_template->assign('content', $this->display_entry($data, false));
        return $this->_template->fetch('admin.preview.html');
    }

    /**
     * pmxSiteup_admin::add()
     *
     * @param array $data
     * @return
     */
    function add($data = array())
    {
        $data = array_merge($this->_defaults, $data);

        $data['date'] = $this->convert_date($data['date']);

        if ($data['preview']) {
            $preview = $this->preview($data, _SITEUP_PREVIEW);
        } else {
            $preview = false;
        }

        $editor = load_class('Textarea');
        if (!$this->_config['wysiwyg']) {
            $editor->setWysiwyg(false);
        }
        if (!$editor->getWysiwyg()) {
            $data['updates'] = mxUndoNltobr($data['updates']);
            $data['bugs'] = mxUndoNltobr($data['bugs']);
        }

        $this->_tabs[_SITEUP_ADD] = '';

        $this->_template->assign($data);
        $this->_template->assign('caption', _SITEUP_ADD);
        $this->_template->assign('preview', $preview);
        $this->_template->assign('op', $this->_module_name . '/add/send');
        $this->_template->assign('module_name', $this->_module_name);
        $this->_template->assign('editor', $editor);
        $content = $this->_template->fetch('admin.mainform.html');

        $this->_template->assign('form', $content);
        $content = $this->_template->fetch('admin.add.html');

        return $this->adminpage($content);
    }

    /**
     * pmxSiteup_admin::add_send()
     *
     * @return
     */
    function add_send()
    {
        if (empty($_POST['files'])) {
            $this->_errors[] = _SITEUP_ERR_SUMMARY;
        }
        if (empty($_POST['updates'])) {
            $this->_errors[] = _SITEUP_ERR_DESCR;
        }

        $date = $this->convert_date($_POST['date']);
        if (!$date) {
            $this->_errors[] = _SITEUP_ERR_DATE;
        }

        $data = array_merge($this->_defaults, $_POST);
        if (empty($data['wysiwyg_updates'])) {
            $data['updates'] = mxNL2BR($data['updates']);
        }
        if (empty($data['wysiwyg_bugs'])) {
            $data['bugs'] = mxNL2BR($data['bugs']);
        }

        switch (true) {
            case $data['preview']:
            case $data['save'] && $this->_errors:
                mxStripSlashes($data);
                return $this->add($data);

            case $data['save']:
                $qry = "INSERT INTO $this->_dbtable (name, `date`, files, updates, bugs) VALUES (
                  '" . mxAddSlashesForSQL($data['aid']) . "',
                  FROM_UNIXTIME(" . $date . "),
                  '" . mxAddSlashesForSQL($data['files']) . "',
                  '" . mxAddSlashesForSQL($data['updates']) . "',
                  '" . mxAddSlashesForSQL($data['bugs']) . "'
              )";

                sql_query($qry);
                return mxRedirect(adminUrl($this->_module_name), _CHANGESAREOK);

            default: // cancel
                return mxRedirect(adminUrl($this->_module_name));
        }
    }

    /**
     * pmxSiteup_admin::edit()
     *
     * @param array $data
     * @return
     */
    function edit($data = array())
    {
        if ($data) {
            $data['date'] = $this->convert_date($data['date']);
            $preview_caption = _SITEUP_PREVIEW;
        } else {
            $id = (isset($_GET['id'])) ? intval($_GET['id']) : 0;
            $result = sql_query("SELECT id, name AS aid, UNIX_TIMESTAMP(`date`) AS date, files, updates, bugs FROM $this->_dbtable WHERE id=" . intval($id));
            $data = sql_fetch_assoc($result);
            $preview_caption = _SITEUP_ORIGINAL;
        }
        $this->_tabs[_SITEUP_MODIFY] = '';

        if (!$data) {
            $this->_errors[] = _SITEUP_ERR_NODATA;
            return $this->adminpage('');
        }

        $data = array_merge($this->_defaults, $data);

        $preview = $this->preview($data, $preview_caption);

        $editor = load_class('Textarea');
        if (!$this->_config['wysiwyg']) {
            $editor->setWysiwyg(false);
        }
        if (!$editor->getWysiwyg()) {
            $data['updates'] = mxUndoNltobr($data['updates']);
            $data['bugs'] = mxUndoNltobr($data['bugs']);
        }

        $this->_template->assign($data);
        $this->_template->assign('caption', _SITEUP_MODIFY);
        $this->_template->assign('preview', $preview);
        $this->_template->assign('op', $this->_module_name . '/edit/send');
        $this->_template->assign('module_name', $this->_module_name);
        $this->_template->assign('editor', $editor);
        $form = $this->_template->fetch('admin.mainform.html');

        $this->_template->assign('form', $form);
        $content = $this->_template->fetch('admin.edit.html');

        return $this->adminpage($content);
    }

    /**
     * pmxSiteup_admin::edit_send()
     *
     * @return
     */
    function edit_send()
    {
        if (empty($_POST['files'])) {
            $this->_errors[] = _SITEUP_ERR_SUMMARY;
        }

        if (empty($_POST['updates'])) {
            $this->_errors[] = _SITEUP_ERR_DESCR;
        }

        $date = $this->convert_date($_POST['date']);
        if (!$date) {
            $this->_errors[] = _SITEUP_ERR_DATE;
        }

        $data = array_merge($this->_defaults, $_POST);
        if (empty($data['wysiwyg_updates'])) {
            $data['updates'] = mxNL2BR($data['updates']);
        }
        if (empty($data['wysiwyg_bugs'])) {
            $data['bugs'] = mxNL2BR($data['bugs']);
        }

        switch (true) {
            case $data['preview']:
            case $data['save'] && $this->_errors:
                mxStripSlashes($data);
                return $this->edit($data);

            case $data['save']:

                $qry = "UPDATE $this->_dbtable SET
                          name='" . mxAddSlashesForSQL($data['aid']) . "',
                          files='" . mxAddSlashesForSQL($data['files']) . "',
                          updates='" . mxAddSlashesForSQL($data['updates']) . "',
                          bugs='" . mxAddSlashesForSQL($data['bugs']) . "',
                          `date`=FROM_UNIXTIME(" . $date . ")
                        WHERE id=" . intval($data['id']);
                sql_query($qry);
                return mxRedirect(adminUrl($this->_module_name), _CHANGESAREOK);

            default: // cancel
                return mxRedirect(adminUrl($this->_module_name));
        }
    }

    /**
     * pmxSiteup_admin::del()
     *
     * @return
     */
    function del()
    {
        $id = (isset($_GET['id'])) ? intval($_GET['id']) : 0;
        $result = sql_query("SELECT id, name AS aid, UNIX_TIMESTAMP(`date`) AS date, files, updates, bugs FROM $this->_dbtable WHERE id=" . intval($id));
        $data = sql_fetch_assoc($result);

        $this->_tabs[_SITEUP_DELETE] = '';

        $preview = $this->preview($data);
        $op = $this->_module_name . '/del/send';
        $this->_template->assign('module_name', $this->_module_name);
        $this->_template->assign(compact('id', 'op', 'preview'));
        $content = $this->_template->fetch('admin.delete.html');
        return $this->adminpage($content);
    }

    /**
     * pmxSiteup_admin::del_send()
     *
     * @return
     */
    function del_send()
    {
        $id = (isset($_POST['id'])) ? intval($_POST['id']) : 0;
        if ($id && isset($_POST['submit'][1]) && !isset($_POST['submit'][0])) {
            sql_query("DELETE FROM $this->_dbtable WHERE id=" . intval($id));
            return mxRedirect(adminUrl($this->_module_name), _DELETEAREOK);
        }
        return mxRedirect(adminUrl($this->_module_name), _DELETENOTOK);
    }
}

$tmp = new pmxSiteup_admin($op);
$tmp = null;

?>