<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 540 $
 * $Author: PragmaMx $
 * $Date: 2022-06-24 19:21:38 +0200 (Fr, 24. Jun 2022) $
 *
 * Based on an earlier Idea for phpNuke 4.4 from Jeff Lambert (jeffx@ican.net)
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * pmxSiteup
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2011
 * @version $Id: class.siteupdate.php 540 2022-06-24 17:21:38Z PragmaMx $
 * @access public
 */
class pmxSiteup {
    protected $_module_name = '';
    protected $_dbprefix = '';
    protected $_dbtable = '';

    protected $_messages = array();
    protected $_errors = array();

    protected $_config = array();
    protected $_template;
    protected $_tabs;

    protected $_is_admin = false;

    /**
     * pmxSiteup::__construct()
     */
    protected function __construct()
    {
        global $prefix;

        $this->_module_name = basename(dirname(dirname(__FILE__)));
        mxGetLangfile($this->_module_name);

        include(dirname(dirname(__FILE__)) . DS . 'includes' . DS . 'settings.php');
        $this->_config = $sup_config;

        $this->_is_admin = mxGetAdminPref('radminsuper', 'radmincontent', 'radminarticle');

        $this->_dbprefix = $prefix;
        $this->_dbtable = $this->_dbprefix . '_siteupdate';

        $this->_tabs = array(/* Standardtabs */
            _SITEUP_ENTRIES => adminUrl($this->_module_name),
            _SITEUP_ADD => adminUrl($this->_module_name, 'add'),
            );

        /* modulspezifisches Stylesheet einbinden */
        pmxHeader::add_style(PMX_MODULES_PATH . $this->_module_name . '/style/style.css');

        /* Template initialisieren */
        $this->_template = load_class('Template');
        $this->_template->init_path(__FILE__);
        $this->_template->assign($this->_config);
        $this->_template->assign('is_admin', $this->_is_admin);
    }

    /**
     * pmxSiteup::display_entry()
     *
     * @param mixed $data
     * @param boolean $adminmode
     * @return
     */
    protected function display_entry($data, $adminmode = true)
    {
        if (!is_array($data)) {
            return false;
        }

        extract($data);

        $files = mxPrepareToDisplay($files);
        $updates = mxPrepareToDisplay($updates);
        $bugs = mxPrepareToDisplay($bugs);
        $date = mx_strftime(_SHORTDATESTRING, ((empty($date)) ? time() : $date)); #_DATESTRING
        $links = array();
        if ($this->_is_admin && $adminmode) {
            $links['edit'] = array('href' => adminUrl($this->_module_name, 'edit', 'id=' . $id), 'caption' => _SITEUP_MODIFY);
            $links['delete'] = array('href' => adminUrl($this->_module_name, 'del', 'id=' . $id), 'caption' => _SITEUP_DELETE);
        }
		$preview=isset($preview)?$preview:"";
        $data = compact('files', 'updates', 'bugs', 'id', 'aid', 'date', 'preview', 'links');
        // $this->_template->assign('caption', $caption);
        $this->_template->assign($data);
		
        if ($this->_config['classic']) {
            return $this->_template->fetch('show.entry.classic.html');
        } else {
            return $this->_template->fetch('show.entry.html');
        }
    }

    /**
     * pmxSiteup::list_entries()
     *
     * @return
     */
    protected function list_entries()
    {
        if (MX_MODULE == 'admin') {
            $link = adminUrl($this->_module_name);
        } else {
            $link = 'modules.php?name=' . $this->_module_name;
        }

        /* aktuelle Seite */
        if (isset($_GET['page'])) {
            $currentpage = intval($_GET['page']);
        }
        if (empty($currentpage)) {
            $currentpage = 1;
        }

        /* Startwert in der Artikel-liste */
        $per_page = intval($this->_config['limit']);
        $offset = ($currentpage - 1) * $per_page;

        $result = sql_query("SELECT COUNT(id) FROM $this->_dbtable");
        list($count_rows) = sql_fetch_row($result);

        $pagination = $this->pagination($link, $count_rows, $per_page, $currentpage);

        $result = sql_query("SELECT id, name AS aid, UNIX_TIMESTAMP(`date`) AS date, files, updates, bugs
                          FROM $this->_dbtable
                          ORDER BY `date` DESC
                          LIMIT " . $offset . "," . $per_page);

        $rows = array();
        while ($row = sql_fetch_assoc($result)) {
            $rows[] = $this->display_entry($row);
        }

        $this->_template->assign('rows', $rows);
        $this->_template->assign('pagination', $pagination);
        $this->_template->assign('module_name', $this->_module_name);

        return $this->_template->fetch('list.entries.html');

        return $content;
    }

    /**
     * pmxSiteup::pagination()
     *
     * @param string $link // The page we are linking to
     * @param integer $count_rows // Total number of items (database results)
     * @param integer $per_page // Max number of items you want shown per page
     * @param integer $currentpage // The current page being viewed
     * @param integer $items // Number of "digit" links to show before/after the currently viewed page
     * @return string
     */
    protected function pagination($link, $count_rows, $per_page, $currentpage, $items = 0)
    {
        if (!intval($count_rows) || !intval($per_page)) {
            return '';
        }

        $pages = ceil($count_rows / $per_page);
        if ($pages <= 1) {
            return '';
        }

        if (!intval($items)) {
            $items = ($GLOBALS['index']) ? 4 : 6;
        }

        $start = 1;
        $end = $pages + 1;

        switch (true) {
            case !$currentpage:
            case intval($currentpage) < 1:
                $currentpage = 1;
                break;
            case $currentpage > $pages:
                $currentpage = $pages;
                break;
        }

        if ($pages > $items) {
            $start = $currentpage - intval(($items -1) / 2);
            if ($start < 1) {
                $start = 1;
            }
            $end = $start + $items;
            if ($end > $pages) {
                $end = $pages + 1;
            }
            if ($end == ($pages + 1)) {
                $start = $end - $items;
            }
        }
        $backlink = ($currentpage > 1) ? ' href="' . $link . '&amp;page=' . ($currentpage - 1) . '"' : '';
        $nextlink = ($currentpage < $pages) ? ' href="' . $link . '&amp;page=' . ($currentpage + 1) . '"' : '';

        $out = '<div class="pagination">';
        $out .= '<span class="counter">' . sprintf(_PAGEOFPAGES, $currentpage, $pages) . '</span>';

        if ($start > 1) {
            $out .= '<a href="' . $link . '" title="' . _GOTOPAGEFIRST . '">1 <span class="arrows">&laquo;</span></a>';
            if ($start > 2) {
                $out .= '<span class="points">..</span>';
            }
        }
        for ($i = $start; $i < $end; $i++) {
            switch ($i) {
                case $currentpage:
                    $out .= '<span class="current">' . $i . '</span>';
                    break;
                case 1:
                    $out .= '<a href="' . $link . '" title="' . _GOTOPAGEFIRST . '">1</a>';
                    break;
                case $pages:
                    $out .= '<a href="' . $link . '&amp;page=' . $i . '" title="' . _GOTOPAGELAST . '">' . $i . '</a>';
                    break;
                default:
                    $out .= '<a href="' . $link . '&amp;page=' . $i . '" title="' . _GOTOPAGE . ": " . $i . '">' . $i . '</a>';
            }
        }
        if ($end < $pages + 1) {
            if ($end < $pages) {
                $out .= '<span class="points">..</span>';
            }
            $out .= '<a href="' . $link . '&amp;page=' . $pages . '" title="' . _GOTOPAGELAST . '"><span class="arrows">&raquo;</span> ' . $pages . '</a>';
        }

        $out .= "</div>";

        return $out;
    }

    /**
     * pmxSiteup::convert_date()
     * Konvertiert ein String-Datum, welches vom Datepicker kommt in
     * einen Timestamp
     *
     * @param mixed $date
     * @return integer timestamp
     */
    protected function convert_date($date)
    {
        switch (true) {
            case is_numeric($date):
                return $date;
            case empty($date):
            case !is_scalar($date):
                return 0;
            default:
                return mkfromstrptime($date, _SHORTDATESTRING);
        }
    }
}

?>