<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 584 $
 * $Author: PragmaMx $
 * $Date: 2022-08-29 21:51:21 +0200 (Mo, 29. Aug 2022) $
 *
 * 
 */

defined('mxMainFileLoaded') or die('access denied');



class pmxStatisticmod {
    protected $_module_name = '';
    protected $_dbprfx = '';
    protected $_config = array();
    protected $_template;
    protected $_tabs;
    protected $_imagepath;
	protected $_isBackend=false;

    protected $_is_admin = false;

    /**
     * pmxStatisticmod::__construct()
     */
    public function __construct()
    {
		global $prefix;
 		$config=array("content"=>1,
			'total'=>1,
			'visitors'=>1,
			'periods'=>1,
			'browsers'=>1,
			'os'=>1,
			'miscstats'=>1,
			'interna'=>1,
			'content'=>1,
			
		);
		
		$cf=load_class("Config","Statistics");		
		$cfstat=$cf->getSection("Statistics");
		$config=array_merge($config,$cfstat);
		$cfstat=$cf->setSection("Statistics",$config);
 
		$this->_dbprfx = $prefix;
		$this->_config = $cf->getSection("Statistics");
		$this->_is_admin = MX_IS_ADMIN;
        $this->_module_name = basename(dirname(dirname(__FILE__)));
        mxGetLangfile($this->_module_name);
		
		if (defined('mxAdminFileLoaded'))  $this->_isBackend=true;
		
        $this->_imagepath = 'modules/' . $this->_module_name . '/images/';	
        $this->_template = load_class('Template');
        $this->_template->init_path(__FILE__);	
		$this->_template->assign('is_admin', $this->_is_admin);		
		
		
	}
	
	public function main($act)
	{
        global $prefix;


        /* modulspezifisches Stylesheet einbinden */
        pmxHeader::add_style(PMX_MODULES_PATH . $this->_module_name . '/style/style.css');

        /* Template initialisieren */

        $pagetitle = _STAT_STATS;

        settype($_GET['year'], 'integer');
        settype($_GET['month'], 'integer');
        settype($_GET['day'], 'integer');
        settype($_GET['op'], 'string');
		$act=isset($_GET['act'])?$_GET['act']:"";

        //$tabs = array(/* Standardtabs */
        $tabs['contents'] = _STAT_MISCS;
         if ($this->_config['visitors']==1) $tabs['visitors'] = _STAT_VISITORS;
         if ($this->_config['periods']==1)  $tabs['periods'] = _STAT_PERIODS;
        //    );

        /* header.php hier bereits einbinden, damit dieser Zugriff gleich angezeigt wird */
        include('header.php');
        switch ($act) {

            case "visitors":
                $content = $this->visitors();
                $currenttab = 'visitors';
                break;

            case "periods":
                $content = $this->periods();
                $currenttab = 'periods';
                break;

            case "yearly":
                $content = $this->yearly();
                $currenttab = 'periods';
                break;

            case "monthly":
                $content = $this->monthly();
                $currenttab = 'periods';
                break;

            case "daily":
                $content = $this->daily();
                $currenttab = 'periods';
                break;
            case "contents":
            default:
                $content = $this->contents();
                $currenttab = 'contents';
                break;
        }
		
        $this->_template->assign('module_name', $this->_module_name);
        $this->_template->assign('tabs', $tabs);
        $this->_template->assign('today', mx_strftime(_DATESTRING));
        $this->_template->assign('current', $currenttab);
        $this->_template->assign('content', $content);

        $this->_template->display('page.html');
        include('footer.php');
    }

    /**
     * contents()
     *
     * @return
     */
     function contents()
    {
        global $user_prefix;
		
        $blocks = $this->get_contentstat();
		//mxDebugFuncVars($blocks);
        $total = pmx_number_format(pmx_get_totalhits());
        $startdate = mx_strftime(_DATESTRING, pmx_get_startdate());
        $tops = $this->get_tops();

        $this->_template->assign(compact('total', 'startdate', 'blocks', 'tops'));
        return $this->_template->fetch('contents.html');
    }

    /**
     * visitors()
     *
     * @return string
     */
    function visitors()
    {
        if ($this->_config['visitors'] == 0 && !$this->_isBackend) {
            return "";
        }

        $startdate = mx_strftime(_DATESTRING, $this->get_counter_startdate());
        $blocks = $this->get_visitorstat();
        $_out = sprintf(_STAT_VISITORSINCE, $startdate);
        if (count($blocks) > 0) {
            $_items = [];
            foreach ($blocks as $value) {
                $_items[] = array($value['title'], $value['content']);
            }
            $_out .= '<br>' . pmxDom::addTabs(["items" => $_items]);
        }
        return $_out;
    }

    /**
     * periods()
     *
     * @return
     */
     function periods()
    {
		if ($this->_config['periods']==0 and !$this->_isBackend) return false;
        $now = getdate();
        $startdate = mx_strftime(_DATESTRING, pmx_get_startdate());
        $years = $this->get_yearly();
        $months = $this->get_monthly($now['year']);
        $days = $this->get_daily($now['year'], $now['mon']);
        $hours = $this->get_hourly($now['year'], $now['mon'], $now['mday']);

        $this->_template->assign(compact('startdate', 'years', 'months', 'days', 'hours'));
        return $this->_template->fetch('periods.html');
    }

    /**
     * yearly()
     *
     * @param mixed $year
     * @return
     */
     function yearly()
    {
        extract($_GET);
        return $this->get_monthly($year);
    }

    /**
     * monthly()
     *
     * @param mixed $year
     * @param mixed $month
     * @return
     */
     function monthly()
    {
        extract($_GET);
        return $this->get_daily($year, $month);
    }

    /**
     * daily()
     *
     * @param mixed $year
     * @param mixed $month
     * @param mixed $day
     * @return
     */
     function daily()
    {
        extract($_GET);
        return $this->get_hourly($year, $month, $day);
    }

    /**
     * get_yearly()
     *
     * @return
     */
     function get_yearly()
    {
        $result = sql_query("SELECT SUM(hits) AS total
                            FROM `{$this->_dbprfx}_stats`
                            WHERE hits > 0
                              AND " . $this->sql_from_startdate() . "");
        list($total) = sql_fetch_row($result);
        $result = sql_query("SELECT year, SUM(hits) AS hits
                        FROM `{$this->_dbprfx}_stats`
                        WHERE " . $this->sql_from_startdate() . "
                        GROUP BY year
                        ORDER BY year");
        $summe = 0;
        $items = array();
        while (list($year, $hits) = sql_fetch_row($result)) {
            settype($hits, 'integer');
            $summe += $hits;
            $percent = ($hits) ? round(100 * $hits / $total) : 0;
			$link = ($this->_isBackend)?
				'<a href="admin.php?op=' . $this->_module_name . '&amp;act=yearly&amp;year=' . $year . '">' . $year . '</a>':
				'<a href="index.php?name=' . $this->_module_name . '&amp;act=yearly&amp;year=' . $year . '">' . $year . '</a>';
            $items[] = array(/* Daten für Template */
                'caption' => $link,
                'progressbar' => $this->percbar($percent),
                'value' => pmx_number_format($hits),
                'percent' => $percent,
                );
        }

        $this->_template->assign('title', _STAT_YEARLY);
        $this->_template->assign('table_caption', _YEAR);
        $this->_template->assign('items', $items);
        $this->_template->assign('summe', pmx_number_format($summe));
        return $this->_template->fetch('statsblock3.html');
    }

    /**
     * get_monthly()
     *
     * @param mixed $year
     * @return
     */
     function get_monthly($year)
    {
        $result = sql_query("SELECT SUM(hits) AS total
                            FROM `{$this->_dbprfx}_stats`
                            WHERE hits > 0
                              AND " . $this->sql_from_startdate() . "
                              AND year=" . intval($year));
        list($total) = sql_fetch_row($result);

        $result = sql_query("SELECT month, SUM(hits) AS hits, year
                        FROM `{$this->_dbprfx}_stats`
                        WHERE " . $this->sql_from_startdate() . "
                          AND year=" . intval($year) . "
                        GROUP BY year, month
                        ORDER BY month;");

        $summe = 0;
        $items = array();
        while (list($month, $hits) = sql_fetch_row($result)) {
            settype($hits, 'integer');
            $summe += $hits;
            $percent = ($hits) ? round(100 * $hits / $total) : 0;
			$link = ($this->_isBackend)?
				'<a href="admin.php?op=' . $this->_module_name . '&amp;act=monthly&amp;year=' . $year .  '&amp;month=' . $month . '">' . $this->translate_month($month) . '</a>':
				'<a href="modules.php?name=' . $this->_module_name . '&amp;act=monthly&amp;year=' . $year . '&amp;month=' . $month . '">' . $this->translate_month($month) . '</a>';
            $items[] = array(/* Daten für Template */
                'caption' => $link,
                'progressbar' => $this->percbar($percent),
                'value' => pmx_number_format($hits),
                'percent' => $percent,
                );
        }

        $this->_template->assign('title', _STAT_MONTLY . ' ' . $year);
        $this->_template->assign('table_caption', _UMONTH);
        $this->_template->assign('items', $items);
        $this->_template->assign('summe', pmx_number_format($summe));
        return $this->_template->fetch('statsblock3.html');
    }

    /**
     * get_daily()
     *
     * @param mixed $year
     * @param mixed $month
     * @return
     */
     function get_daily($year, $month)
    {
        $result = sql_query("SELECT SUM(hits) AS total
                            FROM `{$this->_dbprfx}_stats`
                            WHERE hits > 0
                              AND " . $this->sql_from_startdate() . "
                              AND year=" . intval($year) . "
                              AND month=" . intval($month));
        list($total) = sql_fetch_row($result);

        $result = sql_query("SELECT `date` AS day, SUM(hits) AS hits
                        FROM `{$this->_dbprfx}_stats`
                        WHERE " . $this->sql_from_startdate() . "
                          AND year = " . intval($year) . "
                          AND month = " . intval($month) . "
                        GROUP BY year, month, `date`
                        ORDER BY `date`");

        $summe = 0;
        $items = array();
        while (list($day, $hits) = sql_fetch_row($result)) {
            settype($hits, 'integer');
            $summe += $hits;
            $percent = ($hits) ? round(100 * $hits / $total) : 0;
			$link = ($this->_isBackend)?
				'admin.php?op='.$this->_module_name :
				'modules.php?name='.$this->_module_name;
            $items[] = array(/* Daten für Template */
                'caption' => '<a href="'.  $link . '&amp;act=daily&amp;year=' . $year . '&amp;month=' . $month . '&amp;day=' . $day . '">' . sprintf('%02u', $day) . '</a>',
                'progressbar' => $this->percbar($percent),
                'value' => pmx_number_format($hits),
                'percent' => $percent,
                );
        }

        $this->_template->assign('title', _STAT_DAILY . ' ' . $this->translate_month($month) . ', ' . $year);
        $this->_template->assign('table_caption', _DATE);
        $this->_template->assign('items', $items);
        $this->_template->assign('summe', pmx_number_format($summe));
        return $this->_template->fetch('statsblock3.html');
    }

    /**
     * get_hourly()
     *
     * @param mixed $year
     * @param mixed $month
     * @param mixed $day
     * @return
     */
     function get_hourly($year, $month, $day)
    {
        $result = sql_query("SELECT SUM(hits) AS total
                            FROM `{$this->_dbprfx}_stats`
                            WHERE year=" . intval($year) . "
                              AND month=" . intval($month) . "
                              AND `date`=" . intval($day));
        list ($total) = sql_fetch_row($result);

        $summe = 0;
        $items = array();
        for ($k = 0; $k <= 23; $k++) {
            $result = sql_query("SELECT hits
                                FROM `{$this->_dbprfx}_stats`
                                WHERE year=" . intval($year) . "
                                  AND month=" . intval($month) . "
                                  AND `date`=" . intval($day) . "
                                  AND hour=" . $k . "");
            list($hits) = sql_fetch_row($result);
            settype($hits, 'integer');
            $summe += $hits;
            $percent = ($hits) ? round(100 * $hits / $total) : 0;
            $hour = sprintf('%02u:00 - %02u:59', $k, $k);

            $items[] = array(/* Daten für Template */
                'caption' => $hour,
                'progressbar' => $this->percbar($percent),
                'value' => pmx_number_format($hits),
                'percent' => $percent,
                );
        }

        $this->_template->assign('title', _STAT_HOURLY . ' ' . mx_strftime(_DATESTRING, mktime(0, 0, 0, $month, $day, $year)));
        $this->_template->assign('table_caption', _HOUR);
        $this->_template->assign('items', $items);
        $this->_template->assign('summe', pmx_number_format($summe));
        return $this->_template->fetch('statsblock3.html');
    }

    /**
     * percbar()
     *
     * @param mixed $percent
     * @return
     */
    private function percbar($percent)
    {
        $this->_template->assign('percent', floatval($percent));
        return $this->_template->fetch('percbar.html');
    }

    /**
     * translate_month()
     *
     * @param mixed $month
     * @return
     */
    private function translate_month($month)
    {
        switch (intval($month)) {
            case 1: return _JANUARY;
            case 2: return _FEBRUARY;
            case 3: return _MARCH;
            case 4: return _APRIL;
            case 5: return _MAY;
            case 6: return _JUNE;
            case 7: return _JULY;
            case 8: return _AUGUST;
            case 9: return _SEPTEMBER;
            case 10: return _OCTOBER;
            case 11: return _NOVEMBER;
            case 12: return _DECEMBER;
        }
    }

    /**
     * pmxStatisticmod::get_counter_total()
     *
     * @return
     */
    private function get_counter_total()
    {
        $result = sql_query("SELECT count
                            FROM `{$this->_dbprfx}_counter`
                            WHERE type='total'
                              AND var='hits'");
        list($total) = sql_fetch_row($result);
        return intval($total);
    }

    /**
     * pmxStatisticmod::get_counter_startdate()
     *
     * @return
     */
    private function get_counter_startdate()
    {
        $result = sql_query("SELECT count
                            FROM `{$this->_dbprfx}_counter`
                            WHERE type='reset'
                              AND var='date'");
        list($start) = sql_fetch_row($result);
        return intval($start);
    }

    /**
     * pmxStatisticmod::get_visitorstat()
     *
     * @return
     */
    private function get_visitorstat()
    {
		$blocks=array();
		
        $parts = array(/* Array vorher zur Sortierung erstellen */
            'browser' => array('caption' => _STAT_BROWSERS, 'items' => array(), 'sum' => 0),
            'robot' => array('caption' => _STAT_BOTS, 'items' => array(), 'sum' => 0),
            'os' => array('caption' => _STAT_OPERATINGSYS, 'items' => array(), 'sum' => 0),
            // 'referer' => array('caption' => _SEARCHENGINES, 'items' => array(), 'sum'=>0),
            );

        $total = $this->get_counter_total();

        $result = sql_query("SELECT type, var, count, icon
                            FROM `{$this->_dbprfx}_counter`
                            WHERE type NOT IN ('total','reset')
                            ORDER BY type DESC, count DESC");
        while (list($type, $var, $count, $icon) = sql_fetch_row($result)) {
            $percent = ($total) ? round(100 * $count / $total, 2) : 0;
            $parts[$type]['items'][$var] = array(/* Daten */
                'percent' => $percent,
                'count' => $count,
                'value' => pmx_number_format($count),
                'icon' => $type . '/' . $icon,
                );
        }

        $blocks = array();
        foreach ($parts as $type => $xparts) {
            if ($xparts['items']) {
                $items = array();
                if (array_key_exists('unknown', $xparts['items'])) {
                    /* unbekannte Browser ans Ende stellen */
                    $tmp = $xparts['items']['unknown'];
                    unset($xparts['items']['unknown']);
                    $xparts['items'][_STAT_OTHER] = $tmp;
                } else if (array_key_exists('other', $xparts['items'])) {
                    /* andere Bots ans Ende stellen */
                    $tmp = $xparts['items']['other'];
                    unset($xparts['items']['other']);
                    $xparts['items'][_STAT_OTHER] = $tmp;
                }

                $summe = 0;
                $items = array();
                foreach ($xparts['items'] as $key => $row) {
                    $summe += $row['count'];
                    $items[] = array(/* Daten für Template */
                        'caption' => $key,
                        'progressbar' => $this->percbar($row['percent']),
                        'value' => $row['value'],
                        'percent' => $row['percent'],
                        'icon' => $this->_imagepath . $row['icon'],
                        );
                }

                $title = $xparts['caption'];

                $this->_template->assign('title', $title);
                $this->_template->assign('table_caption', $title);
                $this->_template->assign('items', $items);
                $this->_template->assign('summe', pmx_number_format($summe));
                $blocks[$type] = array('content' => $this->_template->fetch('statsblock1.html'), 'title' => $title);
            }
        }

        return $blocks;
    }

    /**
     * pmxStatisticmod::get_contentstat()
     *
     * @return
     */
     function get_contentstat()
    {
        global $user_prefix;
		$blocks=array();
        /* >>>>>>> Block "Inhalte" <<<<<<<<<<<<< */
	if ($this->_config['miscstats']==1 OR $this->_isBackend  ) 
	{
        $hook = load_class('Hook', 'statistics.content');

        $stat1 = $hook->get(array());
        if ($stat1 && is_array($stat1)) {
            $default = array(/* Standarddatensatz */
                'module' => '',
                'url' => '',
                'icon' => '',
                'caption' => '',
                'value' => 0,
                );

            foreach ($stat1 as $key => $value) {
                $stat1[$key] = array_merge($default, $value);
            }

            $this->_template->assign('title', _STAT_MISCSTATS);
            $this->_template->assign('items', $stat1);
            $blocks['content'] = $this->_template->fetch('statsblock2.html');
        }
	}
        /* >>>>>>> Block "Interna" <<<<<<<<<<<<< */
	if ($this->_config['interna']==1 OR $this->_isBackend  ) {
        $stat2[] = array('case' => true,
            'icon' => 'authors.gif',
            'caption' => _STAT_ACTIVEAUTHORS,
            'query' => "select aid from {$this->_dbprfx}_authors",
            );

        $stat2[] = array('case' => true,
            'icon' => 'users.gif',
            'caption' => _STAT_REGUSERS,
            'query' => "select uid from {$user_prefix}_users where user_stat=1",
            );

        $stat2[] = array('case' => $this->_is_admin,
            'icon' => 'friend.gif',
            'caption' => _STAT_PRIMESS,
            'query' => "select * from {$this->_dbprfx}_priv_msgs",
            );

        $stat2[] = array('case' => $this->_is_admin,
            'icon' => 'waiting.gif',
            'caption' => _STAT_NEWSWAITING1,
            'query' => "select * from {$this->_dbprfx}_queue",
            );

        $items = array();
        foreach ($stat2 as $item) {
            if ($item['case']) {
                $value = sql_num_rows(sql_query($item['query']));
                if ($value) {
                    $items[] = array(/* Daten für Template */
                        'module' => '',
                        'url' => '',
                        'caption' => $item['caption'],
                        'value' => $value,
                        'icon' => $this->_imagepath . $item['icon'],
                        );
                }
            }
        }

        $items[] = array(/* Daten für Template */
            'module' => '',
            'url' => '',
            'caption' => _STAT_PMXVERSION,
            'value' => PMX_VERSION,
            'icon' => $this->_imagepath . 'sections.gif',
            );

        $this->_template->assign('title', _STAT_PERS);
        $this->_template->assign('items', $items);
        $blocks['interna'] = $this->_template->fetch('statsblock2.html');
	}
        return $blocks;
    }

    /**
     * pmxStatisticmod::get_tops()
     *
     * @return
     */
     function get_tops()
    {
        $now = getdate();
        $items = array();

        /* MOSTMONTH */
        $result = sql_query("SELECT year, month, Sum(hits)
                        FROM `{$this->_dbprfx}_stats`
                        WHERE hits > 0
                          AND " . $this->sql_from_startdate() . "
                        GROUP BY year, month;");
        $maxhit = 0;
        $maxyear = $now['year'];
        $maxmonth = $now['mon'];
        while (list($year, $month, $hits) = sql_fetch_row($result)) {
            if ($maxhit < $hits) {
                $maxhit = $hits;
                $maxyear = $year;
                $maxmonth = $month;
            }
        }

        $items[] = array('caption' => _STAT_MOSTMONTH,
            'value' => $this->translate_month($maxmonth) . " " . $maxyear,
            'hits' => pmx_number_format($maxhit));

        /* MOSTDAY */
        $result = sql_query("SELECT year, month, `date` AS day, Sum(hits)
                        FROM `{$this->_dbprfx}_stats`
                        WHERE hits > 0
                          AND " . $this->sql_from_startdate() . "
                        GROUP BY year, month, `date`;");
        $maxhit = 0;
        $maxyear = $now['year'];
        $maxmonth = $now['mon'];
        $maxday = $now['mday'];
        while (list($year, $month, $day, $hits) = sql_fetch_row($result)) {
            if ($maxhit < $hits) {
                $maxhit = $hits;
                $maxyear = $year;
                $maxmonth = $month;
                $maxday = $day;
            }
        }
        $items[] = array('caption' => _STAT_MOSTDAY,
            'value' => mx_strftime(_DATESTRING, mktime(0, 0, 0, $maxmonth, $maxday, $maxyear)),
            'hits' => pmx_number_format($maxhit));

        /* MOSTHOUR */
        $result = sql_query("SELECT year, month, `date` AS day, hour, hits
                            FROM `{$this->_dbprfx}_stats`
                            WHERE hits > 0
                              AND " . $this->sql_from_startdate() . "
                            ORDER BY hits DESC
                            LIMIT 0,1");
        list($year, $month, $day, $hour, $hits) = sql_fetch_row($result);
        $hour = sprintf('%02u:00 - %02u:59', $hour, $hour);
        $items[] = array('caption' => _STAT_MOSTHOUR,
            'value' => $hour . " " . _ON . " " . mx_strftime(_DATESTRING, mktime(0, 0, 0, $month, $day, $year)),
            'hits' => pmx_number_format($hits));

        return $items;
    }

    protected function sql_from_startdate()
    {
        $start = getdate(pmx_get_startdate());
        if ($start) {
            return "STR_TO_DATE(CONCAT(year, '-', month, '-', `date`), '%Y-%c-%e') >= '$start[year]-$start[mon]-$start[mday]'";
        } else {
            return "11111=22222";
        }
    }
}

?>