<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 *
 *
 */

defined('mxMainFileLoaded') or die('access denied');

if (!mxGetAdminPref('radminsurvey')) {
    return mxErrorScreen('Access Denied');
}

$module_name = basename(dirname(__DIR__));
mxGetLangfile($module_name); // äöüß

require_once(PMX_MODULES_DIR . DS . $module_name . DS . 'includes' . DS . 'functions.php');

function poll_form($poll)
{
    global $prefix;

    if (!($poll->id)) {
        // neuer Datensatz
        $action = PMX_MODULE . '/create_ok';
        $button = _CREATEPOLLBUT;
    } else {
        // Ändern
        $action = PMX_MODULE . '/save';
        $button = _SAVECHANGES;
    }

    $stitle = '';
    if ($poll->artid) {
        $result = sql_query("SELECT title FROM ".$prefix."_stories` WHERE `sid` = " . intval($poll->artid));
        list($stitle) = sql_fetch_row($result);
    }

    $data = $poll->get();

    $min = count($data['options']) + 1;
    if ($min < 4) {
        $min = 4;
    }
    for($i = 1; $i <= $min; $i++) {
        if (!isset($data['options'][$i])) {
            $data['options'][$i] = array(/*  */
                'optionText' => '',
                'optionCount' => '',
                'barcolor' => '',
                'fgcolor' => 'inherit',
                );
        }
    }

    /* Template initialisieren */
    $template = load_class('Template');
    $template->init_path(__FILE__);

    /* Daten dem Template zuweisen */
    $template->assign($data);
    $template->assign(compact('action', 'button', 'stitle'));

    /* Template ausgeben */
    return $template->fetch('poll_form.html');
}

function poll_header($op = 'create')
{
    mxIncludeHeader();
    title(_POLLSADMIN);

	pmxHeader::add_jquery('ui/jquery.ui.tooltip.min.js');
	
    ?>
<ul class="tabs-nav">
  <li <?php echo (($op == 'create') ? ' class="current"' : '') ?> ><a href="<?php echo adminUrl(PMX_MODULE) ?>"><?php echo _CREATEPOLL ?></a></li>
  <li <?php echo (($op == 'edit') ? ' class="current"' : '') ?> ><a href="<?php echo adminUrl(PMX_MODULE, 'edit_select') ?>"><?php echo _EDITPOLL ?></a></li>
  <li <?php echo (($op == 'delete') ? ' class="current"' : '') ?> ><a href="<?php echo adminUrl(PMX_MODULE, 'delete') ?>"><?php echo _DELETEPOLLS ?></a></li>
</ul>
<div class="tabs-panel">
<?php

}

function poll_footer()
{
    echo '</div>'; // ende tabs-panel
    mxIncludeFooter();
}

function poll_create()
{
    $poll = new pmx_poll(0);

    poll_header('create');

    echo "
    <fieldset>
      <legend>" . _CREATEPOLL . "</legend>
      " . poll_form($poll) . "
      </fieldset>";
    poll_footer();
}

function poll_delete()
{
    global $prefix;

    $result = sql_query("SELECT pollID, pollTitle, planguage, pollactive FROM `{$prefix}_poll_desc` ORDER BY timeStamp");
    $rows = array(0 => '', 1 => '', 2 => '');

    while ($object = sql_fetch_array($result)) {
        $var = intval($object['pollactive']);
        $lang = (empty($object['planguage'])) ? _ALL : mxGetLanguageString($object['planguage']);
        $rows[$var] .= "<tr>
            <td><input type=\"radio\" name=\"id\" value=\"" . $object['pollID'] . "\" /></td>
            <td>" . $object['pollTitle'] . "</td>
            <td>" . $lang . "</td>
          </tr>";
    }

    poll_header('delete');

    echo "
    <p>" . _CHOOSEPOLL . "</p>
    <form action=\"" . adminUrl(PMX_MODULE) . "\" method=\"post\">
    <input type=\"hidden\" name=\"op\" value=\"" . PMX_MODULE . "/delete_ok\" />
    <table class=\"list\">";
    if ($rows[1]) {
        echo "<tr><th colspan=3>" . _ACTIVE . "</th></tr>" . $rows[1];
    }
    if ($rows[2]) {
        echo "<tr><th colspan=3>" . _POLLCLOSED . "</th></tr>" . $rows[2];
    }
    if ($rows[0]) {
        echo "<tr><th colspan=3>" . _INACTIVE . "</th></tr>" . $rows[0];
    }
    echo "
    </table>
    <p class=\"warning\">" . _POLLDELWARNING . "</p>
    <input type=\"submit\" value=\"" . _DELETE . "\" />
    </form>";
    poll_footer();
}

function poll_delete_ok()
{
    global $id, $prefix;
    sql_query("DELETE FROM `{$prefix}_poll_check` WHERE `pollID` =" . intval($id));
    sql_query("DELETE FROM `{$prefix}_pollcomments` WHERE `pollID` =" . intval($id));
    sql_query("DELETE FROM `{$prefix}_poll_data` WHERE `pollID` =" . intval($id));
    sql_query("DELETE FROM `{$prefix}_poll_desc` WHERE `pollID` =" . intval($id));
    return mxRedirect(adminUrl(PMX_MODULE, 'edit_select'));
}

function polledit_select()
{
    global $prefix;

    $rows = array(0 => '', 1 => '', 2 => '');
    $result = sql_query("SELECT pollID, pollTitle, planguage, pollactive FROM `{$prefix}_poll_desc` ORDER BY timeStamp");

    while ($object = sql_fetch_array($result)) {
        $var = intval($object['pollactive']);
        $lang = (empty($object['planguage'])) ? _ALL : mxGetLanguageString($object['planguage']);
        $rows[$var] .= "
        <tr>
          <td><a href=\"" . adminUrl(PMX_MODULE, 'edit', array('pollID' => $object['pollID'])) . "\">" . $object['pollTitle'] . "</a></td>
          <td>" . $lang . "</td>
        </tr>";
    }

    poll_header('edit');
    echo "
    <p>" . _CHOOSEPOLLEDIT . "</p>
    <table class=\"list\">";
    if ($rows[1]) {
        echo "<tr><th colspan=2>" . _ACTIVE . "</th></tr>" . $rows[1];
    }
    if ($rows[1]) {
        echo "<tr><th colspan=2>" . _POLLCLOSED . "</th></tr>" . $rows[2];
    }
    if ($rows[0]) {
        echo "<tr><th colspan=2>" . _INACTIVE . "</th></tr>" . $rows[0];
    }
    echo "
    </table>";
    poll_footer();
}

function polledit()
{
    switch (true) {
        case empty($_REQUEST['pollID']):
        case !intval($_REQUEST['pollID']):
            return mxErrorScreen(_POLLNOTEXIST);
    }

    $poll = new pmx_poll($_REQUEST['pollID']);

    if (!$poll->exist()) {
        return mxErrorScreen(_POLLNOTEXIST);
    }

    poll_header('edit');
    echo "
    <fieldset>
      <legend>" . _EDITPOLL . ": " . $poll->pollTitle . "</legend>
      " . poll_form($poll) . "
    </fieldset>";
    poll_footer();
}

function poll_db_save($action)
{
    global $prefix;

    settype($_POST['pollID'], 'int');

    /* Standardwerte auslesen und mit einfügen */
    $poll = new pmx_poll($_POST['pollID']);

    $pvs = array_merge($poll->get(), $_POST);
    extract($pvs);

    $barcolordefaults = array("336699", "669933", "339966");

    $sum = 0;
    $newoptions = array();
    $i = 0;
    foreach ($optiontext as $key => $text) {
        /* nur gefüllte Felder berücksichtigen */
        if ($text) {
            $i++;
            $count = intval($optioncount[$key]);
            $sum += $count;

            $color = strtolower(trim($optioncolor[$key], '# '));
            switch (true) {
                case $color:
                    break;
                case !empty($options[$i]['barcolor']):
                    // falls ein alter Wert existiert, diesen verwenden
                    // $options kommt aus dem extract()
                    $color = $options[$i]['barcolor'];
                    break;
                case ($i % 3) === 0:
                    // primitiv gemacht, aber es sind ja nur 3 Standardwerte, wären es mehr, müsste das ne Schleife sein...
                    $color = $barcolordefaults[2];
                    break;
                case ($i % 2) === 0:
                    $color = $barcolordefaults[1];
                    break;
                // case ($i % 1) === 0:
                default:
                    $color = $barcolordefaults[0];
                    break;
            }

            $newoptions[$i] = array(/*  */
                'optionText' => $text,
                'optionCount' => $count,
                'barcolor' => $color,
                );
        }
    }

    $qry = array();
    $fields = array(/*  */
        "`pollTitle`     = '" . mxAddSlashesForSQL($pollTitle) . "'",
        "`planguage`     = '" . mxAddSlashesForSQL($planguage) . "'",
        "`description`   = '" . mxAddSlashesForSQL($description) . "'",
        "`onlyonevote`    = " . ((empty($onlyonevote)) ? 0 : 1),
        "`pollactive`     = " . ((empty($pollactive)) ? 0 : intval($pollactive)),
        "`pollcomments`   = " . ((empty($pollcomments)) ? 0 : intval($pollcomments)),
        "`representative` = " . ((empty($representative)) ? 0 : 1),
        "`artid`          = " . ((empty($artid)) ? 0 : intval($artid)),
        );

    $_ok = false;
    switch ($action) {
        case 'edit':
            $_ok = true;
            // $forwarder = adminUrl(PMX_MODULE, 'edit', array('pollID'=>$pollID));
            $fields[] = "`voters`    = " . $sum;
            $qry[] = "UPDATE `{$prefix}_poll_desc` SET " . implode(', ', $fields) . " WHERE `pollID` = {$pollID}";
            break;
        case 'insert':
            if (!empty($pollTitle) && count($newoptions) > 0) {
                $_ok = true;
            
                // $forwarder = adminUrl(PMX_MODULE, 'edit', array('pollID'=>$pollID));
                $fields[] = "`voters`    = 0";
                $fields[] = "`timeStamp` = " . time();
                $result = sql_query("INSERT INTO `{$prefix}_poll_desc` SET " . implode(', ', $fields));
                $pollID = sql_insert_id();
                if (!$result || !$pollID) {
                    return mxErrorScreen('Database Error: ' . sql_error());
                }
            }
            break;
        default:
            return mxErrorScreen('false action ' . $action);
    }

    $qry[] = "DELETE FROM `{$prefix}_poll_data` WHERE `pollID` = {$pollID}";

    if ($_ok === true) {
        foreach ($newoptions as $key => $option) {
            $text = mxAddSlashesForSQL($option['optionText']);
            $color = mxAddSlashesForSQL($option['barcolor']);
            $count = intval($option['optionCount']);
            $qry[] = "INSERT INTO `{$prefix}_poll_data` SET
            `pollID`      = {$pollID},
            `voteID`      = {$key},
            `optionText`  = '{$text}',
            `barcolor`    = '{$color}',
            `optionCount` = {$count}";
        }
    }

    foreach ($qry as $key => $sql_query) {
        sql_query($sql_query);
    }

    if ($_ok === true) {
        return mxRedirect(adminUrl(PMX_MODULE, $action, array('pollID' => $pollID)));
    }
    // else
    mxIncludeHeader();
    title(_CREATEPOLL);
    echo '<p class="align-center warning">'
        . _POLLERRORDESCRIPTION . '<br><br>'
        . _POLLERRORTITLE . '<br>'
        . _POLLERRORANSWERS . '<br>'
        . _GOBACK . '</p>';
    poll_footer();
    mxIncludeFooter();
}

function removePollSubComments($tid)
{
    global $prefix;
    $result = sql_query("select tid from `{$prefix}_pollcomments` where pid='$tid'");
    $numrows = sql_num_rows($result);
    if ($numrows > 0) {
        while (list($stid) = sql_fetch_row($result)) {
            removePollSubComments($stid);
            sql_query("delete from `{$prefix}_pollcomments` where tid=$stid");
        }
    }
    sql_query("delete from `{$prefix}_pollcomments` where tid=$tid");
}

function RemoveComment($tid, $pollID, $ok = 0)
{
    if ($ok) {
        removePollSubComments($tid);
        return mxRedirect("modules.php?name=" . PMX_MODULE . "&op=results&pollID=$pollID");
    } else {
        include('header.php');

        title(_REMOVECOMMENTS);
        echo '
        	<p class="align-center warning">'
         . _SURETODELCOMMENTS . '<br /><br />
        		 [&nbsp;<a href="modules.php?name=' . PMX_MODULE . '&amp;op=results&amp;pollID=' . $pollID . '" onclick="history.go(-1); return false;">' . _NO . '</a> | <a href="' . adminUrl(PMX_MODULE, 'RemoveComment', array('tid' => $tid, 'pollID' => $pollID, 'ok' => 1)) . '">' . _YES . '</a>&nbsp;]
        	</p>';
        poll_footer();
    }
}

function SelectArticle($artid)
{
    global $prefix;

    $len = 60;

    /* die NEIN Option */
    $sel = ($artid == 0) ? 'selected="selected" class="current"' : '';
    $options[0] = "<option value=\"0\" $sel>- " . _NO . "</option>";

    /* den aktuell zugewiesenen Artikel auf jeden Fall mit anzeigen  */
    if ($artid) {
        $result = sql_query("SELECT sid, title
    		FROM `${prefix}_stories`
        WHERE sid = " . intval($artid));
        if (list($sid, $title) = sql_fetch_row($result)) {
            $options[$sid] = "<option value=\"$sid\" selected=\"selected\" class=\"current\" title=\"" . mxEntityQuotes($title) . "\">" . mxEntityQuotes(substr($title, 0, $len)) . "</option>";
        }
    }
    /* bereits zugewiesene Umfragen ausschliessen */
    $result = sql_query("SELECT artid FROM `{$prefix}_poll_desc` WHERE artid<>''");
    $tmp = "0";
    while (list($sid) = sql_fetch_row($result)) {
        $tmp .= ", $sid";
    }
    /* die neusten 35 Artikel, wird ein älterer benötigt, muss das über den entspr. Artikel gemacht werden */
    $result = sql_query("SELECT sid, title
    		FROM `${prefix}_stories`
        WHERE `time` <= now()
          AND sid NOT IN({$tmp})
    		ORDER BY time desc
        LIMIT 0, 35");
    while (list($sid, $title) = sql_fetch_row($result)) {
        if (strlen($title) > $len) {
            $options[$sid] = "<option value=\"$sid\" title=\"" . mxEntityQuotes($title) . "\">" . mxEntityQuotes(substr($title, 0, $len)) . "...</option>";
        } else {
            $options[$sid] = "<option value=\"$sid\">" . mxEntityQuotes($title) . "</option>";
        }
    }

    return '
    <select name="artid">
      ' . implode(PHP_EOL, $options) . '
    </select>';
}

switch ($op) {
    case PMX_MODULE . "/create_ok":
        poll_db_save('insert');
        break;

    case PMX_MODULE . "/delete":
        poll_delete();
        break;

    case PMX_MODULE . "/delete_ok":
        poll_delete_ok();
        break;

    case PMX_MODULE . "/edit":
        polledit();
        break;

    case PMX_MODULE . "/save":
        poll_db_save('edit');
        break;

    case PMX_MODULE . "/edit_select":
        polledit_select();
        break;

    case PMX_MODULE . "/RemoveComment":
        if (empty($ok)) $ok = 0;
        RemoveComment($tid, $pollID, $ok);
        break;

    case PMX_MODULE . "/create":
    default:
        poll_create();
        break;
}

?>