<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');

/* --------- Konfiguration fuer den Block ----------------------------------- */

/* Maximale Stringlaenge des Theme-Namens im Selectfeld */
$maxwordlen = 20;

/* Maximale Anzahl der Zeilen des Selectfeldes */
$maxselectlines = 6;

/* --------- Ende der Konfiguration ----------------------------------------- */

extract($block['settings'], EXTR_OVERWRITE);

/* der passende Modulname */
$module_name = basename(dirname(__DIR__));

/* Block darf nicht gecached werden und weg... */
$mxblockcache = false;

if (!mxModuleAllowed($module_name)) {
    return;
} //40

//if (!defined('_TTCHANGEIT')) {
//    // mxGetLangfile($module_name);
//}

/* Standardtheme auslesen */
$themes = load_class("Config","pmx.themes");

$to = basename($_SERVER['PHP_SELF']);
// index.php ist auch php_self=modules.php, deswegen hier index.php verwenden, falls $name leer ist
if ($to == 'modules.php' && empty($_REQUEST['name'])) {
    $to = './';
}

$hidden = array();
// die GET Parameter auslesen und (später) zu einem String zusammensetzen
foreach($_GET as $key => $value) {
    if (is_array($value)) continue 1;
    $key = mxSecureValue($key, true);
    // themeselect nicht nochmals anfügen
    if ($key != 'themeselect') { //60
        $args[] = "{$key}={$value}";
        $hidden[] = '<input type="hidden" name="' . $key . '" value="' . $value . '" />';
    }
}

if (isset($_GET['themeselect']) && $_GET['themeselect'] != MX_THEME) {
    if (empty($_GET['themeselect'])) {
        $_GET['themeselect'] = $themes->defaulttheme;
    }
    if (!preg_match('#[^A-Za-z0-9_-]#', $_GET['themeselect'])) {
        if (file_exists(PMX_THEMES_DIR . DS . $_GET['themeselect'] . DS . 'theme.php')) {
            if (MX_IS_USER || MX_IS_ADMIN) {
                mxSessionSetVar('theme', $_GET['themeselect']);
            } else {
                mxSetCookie('theme', $_GET['themeselect'], 1);
            }
            if (isset($args)) {
                $to .= '?' . implode('&', $args);
            } else {
                $to .= '';
            }
            mxRedirect($to, '', 0);
            exit;
        }
    }
}

$themelist = mxGetAvailableThemes();
$options = array();
foreach ($themelist as $th_name => $theme) {
    $selected = ($theme == MX_THEME) ? ' selected="selected" class="current"' : '';
    $title = ($theme == $themes->defaulttheme) ? $theme . ' (' . _TTSITEDEFAULT . ')': $theme;
    $options[$th_name] = '<option value="' . $theme . '" title="' . $title . '"' . $selected . '>' . mxCutString($theme, $maxwordlen, '..', '') . '</option>';
}
$options['#'] = '<option value="" title="' . _TTSITEDEFAULT . ' (' . $themes->defaulttheme . ')">-&nbsp;' . _TTSITEDEFAULT . '</option>';

ksort($options);
if (count($options) < $maxselectlines) {
    $maxselectlines = count($options);
}

/* Wenn der Block links oder rechts positioniert ist, die Breite auf fast 100% stellen */
$width = ($block['position'] == 'l' || $block['position'] == 'r') ? ' style="width: 99%;"' : '';

$content = '
<form action="' . $to . '" method="get">
' . implode("\n", $hidden) . '
<select name="themeselect" size="' . $maxselectlines . '"' . $width . '>
    ' . implode("\n    ", $options) . '
</select>
<input type="submit" value="' . _TTCHANGEIT . '" />
</form>
';

?>