<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 605 $
 * $Author: PragmaMx $
 * $Date: 2022-09-09 15:31:37 +0200 (Fr, 09. Sep 2022) $
 *
 * modified by (c) 2005 by jubilee - http://www.marx-city.de
 */

defined('mxMainFileLoaded') or die('access denied');

include(PMX_SYSTEM_DIR . DS . 'html2fpdf' . DS . 'fpdf.php');

function uguest_backup()
{
    global $prefix, $user_prefix, $module_name, $allow_backup, $owner, $is_own_book;

    if (!$is_own_book || !$allow_backup) {
        guestbook_message(_USERGBNOBACKUP . ' ' . _USERGBNOBACKUP2);
        return;
    }

    ini_set("max_execution_time", "180");
    set_time_limit(180);
    $query = "SELECT gb.wer, gb.gid, gb.date, gb.text, gb.dummy, u.user_avatar, u.uname
                FROM " . $prefix . "_userguest AS gb LEFT JOIN {$user_prefix}_users AS u ON gb.wer = u.uname
                WHERE gb.touser='" . mxAddSlashesForSQL($owner['uname']) . "' ORDER BY gb.gid ASC";
    if (!$result = sql_query($query)) {
        guestbook_message(_GBERRORTXT1 . '  backup-query');
        return;
    }
    $icount = 0;
    while ($eintrag = sql_fetch_assoc($result)) {
        $Textvariable[] = _INPUTNAME . ' ' . $eintrag['uname'] . ' ' . mx_strftime(_XDATESTRING, $eintrag['date']) . " :\n"
         . "------------------------------------------------------------\n"
         . $eintrag['text'] . "\n";
        $icount++;
    }
    if ($icount == 0) {
        $Textvariable = "empty";
    }

    $pdf = new PDF ();
    $pdf->AliasNbPages();
    $pdf->SetCreator('pragmaMx-Usergaestebuch Backupgenerator');
    $pdf->SetAuthor('pragmaMx - Web Content Management System');
    $pdf->SetTitle('Usergaestebuch Backupsatz');
    $pdf->SetSubject('Backup');
    $pdf->AddPage();
    $pdf->SetFont('Times', '', 12);
    if ($Textvariable != "empty") {
        foreach ($Textvariable as $key => $output) {
            pmxDebug::pause();
            $output = de_smiler($output);
            $output = mx_utf8_decode($output);
            pmxDebug::restore();
            $pdf->WriteHTML($output);
            $pdf->Ln(5);
        }
    }
    $pdf->Output('myGuestbook_Backup.pdf', 'D');

    ini_restore("max_execution_time");
}

class PDF extends FPDF {
    // variables of html parser
    var $B;
    var $I;
    var $U;
    var $HREF;
    var $fontList;
    var $issetfont;
    var $issetcolor;

    function PDF($orientation = 'P', $unit = 'mm', $format = 'A4')
    {
        $this->FPDF($orientation, $unit, $format);
        $this->B = 0;
        $this->I = 0;
        $this->U = 0;
        $this->HREF = '';
        $this->fontlist = array("arial", "times", "courier", "helvetica", "symbol");
        $this->issetfont = false;
        $this->issetcolor = false;
    }

    function Header()
    {
        global $module_name;
        $this->Image("modules/" . $module_name . "/pragmamxbanner3.gif", 50, 8, 100);
        $this->Ln(15);
        $this->SetFont('Arial', 'B', 15);
        $this->Cell(0, 5, '', 0, 2, 'C');
        pmxDebug::pause();
        $this->Cell(0, 10, mx_utf8_decode(_USERGB_BACKUPTITLE), 1, 1, 'C');
        pmxDebug::restore();
        $this->Ln(20);
    }

    function Footer()
    {
        $this->SetY(-15);
        $this->SetFont('Arial', 'I', 8);
        $this->Cell(0, 10, 'Page ' . $this->PageNo() . '/{nb}', 0, 0, 'C');
    }

    function WriteHTML($html)
    {
        $html = strip_tags($html, "<b><u><i><a><img><p><br><strong><em><font><tr><blockquote>");
        $html = str_replace("\n", '<br />', $html);
        $a = preg_split('/<(.*)>/U', $html, -1, PREG_SPLIT_DELIM_CAPTURE);
        foreach($a as $i => $e) {
            if ($i % 2 == 0) {
                // Text
                if ($this->HREF)
                    $this->PutLink($this->HREF, $e);
                else
                    $this->Write(5, stripslashes($this->txtentities($e)));
            } else {
                // Tag
                if ($e{0} == '/')
                    $this->CloseTag(strtoupper(substr($e, 1)));
                else {
                    // Extract attributes
                    $a2 = explode(' ', $e);
                    $tag = strtoupper(array_shift($a2));
                    $attr = array();
                    foreach($a2 as $v)
                    if (preg_match('#^([^=]*)=["\']?([^"\']*)["\']?$#', $v, $a3))
                        $attr[strtoupper($a3[1])] = $a3[2];
                    $this->OpenTag($tag, $attr);
                }
            }
        }
    }

    function OpenTag($tag, $attr)
    {
        switch ($tag) {
            case 'STRONG':
                $this->SetStyle('B', true);
                break;
            case 'EM':
                $this->SetStyle('I', true);
                break;
            case 'B':
            case 'I':
            case 'U':
                $this->SetStyle($tag, true);
                break;
            case 'A':
                $this->HREF = $attr['HREF'];
                break;
            case 'IMG':
                if (isset($attr['SRC'])) { // and (isset($attr['WIDTH']) or isset($attr['HEIGHT']))
                    if (!isset($attr['WIDTH']))
                        $attr['WIDTH'] = 15;
                    if (!isset($attr['HEIGHT']))
                        $attr['HEIGHT'] = 15;
                    $this->Image($attr['SRC'], $this->GetX(), $this->GetY(), $this->px2mm($attr['WIDTH']), $this->px2mm($attr['HEIGHT']));
                }
                break;
            case 'TR':
            case 'BLOCKQUOTE':
            case 'BR':
                $this->Ln(5);
                break;
            case 'P':
                $this->Ln(10);
                break;
            case 'FONT':
                if (isset($attr['COLOR']) and $attr['COLOR'] != '') {
                    $coul = $this->hex2dec($attr['COLOR']);
                    $this->SetTextColor($coul['R'], $coul['G'], $coul['B']);
                    $this->issetcolor = true;
                }
                if (isset($attr['FACE']) and in_array(strtolower($attr['FACE']), $this->fontlist)) {
                    $this->SetFont(strtolower($attr['FACE']));
                    $this->issetfont = true;
                }
                break;
        }
    }

    function CloseTag($tag)
    {
        if ($tag == 'STRONG')
            $tag = 'B';
        if ($tag == 'EM')
            $tag = 'I';
        if ($tag == 'B' or $tag == 'I' or $tag == 'U')
            $this->SetStyle($tag, false);
        if ($tag == 'A')
            $this->HREF = '';
        if ($tag == 'FONT') {
            if ($this->issetcolor == true) {
                $this->SetTextColor(0);
            }
            if ($this->issetfont) {
                $this->SetFont('arial');
                $this->issetfont = false;
            }
        }
    }

    function SetStyle($tag, $enable)
    {
        $this->$tag += ($enable ? 1 : -1);
        $style = '';
        foreach(array('B', 'I', 'U') as $s)
        if ($this->$s > 0)
            $style .= $s;
        $this->SetFont('', $style);
    }

    function PutLink($URL, $txt)
    {
        $this->SetTextColor(0, 0, 255);
        $this->SetStyle('U', true);
        $this->Write(5, $txt, $URL);
        $this->SetStyle('U', false);
        $this->SetTextColor(0);
    }

    function hex2dec($couleur = "#000000")
    {
        $R = substr($couleur, 1, 2);
        $rouge = hexdec($R);
        $V = substr($couleur, 3, 2);
        $vert = hexdec($V);
        $B = substr($couleur, 5, 2);
        $bleu = hexdec($B);
        $tbl_couleur = array();
        $tbl_couleur['R'] = $rouge;
        $tbl_couleur['G'] = $vert;
        $tbl_couleur['B'] = $bleu;
        return $tbl_couleur;
    }

    /* conversion pixel -> millimeter in 72 dpi*/
    function px2mm($px)
    {
        return $px * 25.4 / 110;
    }

    function txtentities($html)
    {
        $trans = get_html_translation_table(HTML_ENTITIES);
        $trans = array_flip($trans);
        return strtr($html, $trans);
    }
}

?>